@@ -6,7 +6,6 @@
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
 
 from skbio.util import TestRunner
 
@@ -18,15 +17,16 @@ from skbio.stats.distance import DistanceMatrix
 from skbio.alignment import local_pairwise_align_ssw, TabularMSA
 from skbio.tree import TreeNode, nj
 from skbio.io import read, write
-from skbio._base import OrdinationResults
-
+from skbio.stats.ordination import OrdinationResults
+import skbio.diversity  # noqa
+import skbio.stats.evolve  # noqa
 
 __all__ = ['Sequence', 'DNA', 'RNA', 'Protein', 'GeneticCode',
            'DistanceMatrix', 'local_pairwise_align_ssw', 'TabularMSA',
            'TreeNode', 'nj', 'read', 'write', 'OrdinationResults']
 
 __credits__ = "https://github.com/biocore/scikit-bio/graphs/contributors"
-__version__ = "0.4.1"
+__version__ = "0.5.1"
 
 mottos = [
     # 03/15/2014
