@@ -6,17 +6,15 @@
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
 from warnings import warn
 from itertools import product
 
 import numpy as np
-from future.builtins import range, zip
 
 from skbio.alignment import TabularMSA
 from skbio.alignment._ssw_wrapper import StripedSmithWaterman
 from skbio.sequence import DNA, RNA, Protein
-from skbio.sequence._iupac_sequence import IUPACSequence
+from skbio.sequence import GrammaredSequence
 from skbio.util import EfficiencyWarning
 from skbio.util._decorator import experimental, deprecated
 
@@ -273,9 +271,9 @@ def local_pairwise_align(seq1, seq2, gap_open_penalty,
 
     Parameters
     ----------
-    seq1 : IUPACSequence
+    seq1 : GrammaredSequence
         The first unaligned sequence.
-    seq2 : IUPACSequence
+    seq2 : GrammaredSequence
         The second unaligned sequence.
     gap_open_penalty : int or float
         Penalty for opening a gap (this is substracted from previous best
@@ -323,10 +321,10 @@ def local_pairwise_align(seq1, seq2, gap_open_penalty,
          EfficiencyWarning)
 
     for seq in seq1, seq2:
-        if not isinstance(seq, IUPACSequence):
+        if not isinstance(seq, GrammaredSequence):
             raise TypeError(
-                "`seq1` and `seq2` must be IUPACSequence subclasses, not type "
-                "%r" % type(seq).__name__)
+                "`seq1` and `seq2` must be %r subclasses, not type %r" %
+                (GrammaredSequence.__name__, type(seq).__name__))
 
     if type(seq1) is not type(seq2):
         raise TypeError(
@@ -538,9 +536,9 @@ def global_pairwise_align(seq1, seq2, gap_open_penalty, gap_extend_penalty,
 
     Parameters
     ----------
-    seq1 : IUPACSequence or TabularMSA
+    seq1 : GrammaredSequence or TabularMSA
         The first unaligned sequence(s).
-    seq2 : IUPACSequence or TabularMSA
+    seq2 : GrammaredSequence or TabularMSA
         The second unaligned sequence(s).
     gap_open_penalty : int or float
         Penalty for opening a gap (this is substracted from previous best
@@ -602,11 +600,11 @@ def global_pairwise_align(seq1, seq2, gap_open_penalty, gap_extend_penalty,
 
     for seq in seq1, seq2:
         # We don't need to check the case where `seq` is a `TabularMSA` with a
-        # dtype that isn't a subclass of `IUPACSequence`, this is guaranteed by
-        # `TabularMSA`.
-        if not isinstance(seq, (IUPACSequence, TabularMSA)):
+        # dtype that isn't a subclass of `GrammaredSequence`, this is
+        # guaranteed by `TabularMSA`.
+        if not isinstance(seq, (GrammaredSequence, TabularMSA)):
             raise TypeError(
-                "`seq1` and `seq2` must be IUPACSequence subclasses or "
+                "`seq1` and `seq2` must be GrammaredSequence subclasses or "
                 "TabularMSA, not type %r" % type(seq).__name__)
 
     seq1 = _coerce_alignment_input_type(seq1)
@@ -720,16 +718,24 @@ def local_pairwise_align_ssw(sequence1, sequence2, **kwargs):
             (alignment.target_begin, alignment.target_end_optimal)
         ]
 
+    metadata1 = metadata2 = None
+    if sequence1.has_metadata():
+        metadata1 = sequence1.metadata
+    if sequence2.has_metadata():
+        metadata2 = sequence2.metadata
+
     constructor = type(sequence1)
     msa = TabularMSA([
-        constructor(alignment.aligned_query_sequence),
-        constructor(alignment.aligned_target_sequence)
+        constructor(alignment.aligned_query_sequence, metadata=metadata1,
+                    validate=False),
+        constructor(alignment.aligned_target_sequence, metadata=metadata2,
+                    validate=False)
     ])
 
     return msa, alignment.optimal_alignment_score, start_end
 
 
-@deprecated(as_of="0.4.0", until="0.5.0",
+@deprecated(as_of="0.4.0", until="0.5.2",
             reason="Will be replaced by a SubstitutionMatrix class. To track "
                    "progress, see [#161]"
                    "(https://github.com/biocore/scikit-bio/issues/161).")
@@ -774,7 +780,7 @@ def make_identity_substitution_matrix(match_score, mismatch_score,
 
 
 def _coerce_alignment_input_type(seq):
-    if isinstance(seq, IUPACSequence):
+    if isinstance(seq, GrammaredSequence):
         return TabularMSA([seq])
     else:
         return seq
@@ -900,12 +906,14 @@ def _compute_score_and_traceback_matrices(
 
     # Iterate over the characters in aln2 (which corresponds to the vertical
     # sequence in the matrix)
-    for aln2_pos, aln2_chars in enumerate(aln2.iter_positions(), 1):
+    for aln2_pos, aln2_chars in enumerate(aln2.iter_positions(
+            ignore_metadata=True), 1):
         aln2_chars = str(aln2_chars)
 
         # Iterate over the characters in aln1 (which corresponds to the
         # horizontal sequence in the matrix)
-        for aln1_pos, aln1_chars in enumerate(aln1.iter_positions(), 1):
+        for aln1_pos, aln1_chars in enumerate(aln1.iter_positions(
+                ignore_metadata=True), 1):
             aln1_chars = str(aln1_chars)
 
             # compute the score for a match/mismatch
@@ -1011,15 +1019,23 @@ def _traceback(traceback_matrix, score_matrix, aln1, aln2, start_row,
             raise ValueError(
                 "Invalid value in traceback matrix: %s" % current_value)
 
-    for i in range(aln1_sequence_count):
-        aligned_seq = ''.join(aligned_seqs1[i][::-1])
+    for i, (aligned_seq, original) in enumerate(zip(aligned_seqs1, aln1)):
+        aligned_seq = ''.join(aligned_seq)[::-1]
         constructor = aln1.dtype
-        aligned_seqs1[i] = constructor(aligned_seq)
+        metadata = None
+        if original.has_metadata():
+            metadata = original.metadata
+        aligned_seqs1[i] = constructor(aligned_seq, metadata=metadata,
+                                       validate=False)
 
-    for i in range(aln2_sequence_count):
-        aligned_seq = ''.join(aligned_seqs2[i][::-1])
+    for i, (aligned_seq, original) in enumerate(zip(aligned_seqs2, aln2)):
+        aligned_seq = ''.join(aligned_seq)[::-1]
         constructor = aln2.dtype
-        aligned_seqs2[i] = constructor(aligned_seq)
+        metadata = None
+        if original.has_metadata():
+            metadata = original.metadata
+        aligned_seqs2[i] = constructor(aligned_seq, metadata=metadata,
+                                       validate=False)
 
     return aligned_seqs1, aligned_seqs2, best_score, current_col, current_row
 
