@@ -27,9 +27,6 @@ An empty file consists of only whitespace characters.
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
 from skbio.io import create_format
 
 emptyfile = create_format('<emptyfile>')
