@@ -4,18 +4,16 @@ GenBank format (:mod:`skbio.io.format.genbank`)
 
 .. currentmodule:: skbio.io.format.genbank
 
-GenBank format (GenBank Flat File Format) stores sequence and its annotation
-together. The start of the annotation section is marked by a line beginning
-with the word "LOCUS". The start of sequence section is marked by a line
-beginning with the word "ORIGIN" and the end of the section is marked by a line
-with only "//".
+GenBank format (GenBank Flat File Format) stores sequence and its
+annotation together. The start of the annotation section is marked by
+a line beginning with the word "LOCUS". The start of sequence section
+is marked by a line beginning with the word "ORIGIN" and the end of
+the section is marked by a line with only "//".
 
-The GenBank file usually ends with .gb or sometimes .gbk. The GenBank format
-for protein has been renamed to GenPept. The GenBank (for nucleotide) and
-Genpept are essentially the same format.
-
-An example of a GenBank file can be see here:
-<http://www.ncbi.nlm.nih.gov/Sitemap/samplerecord.html>
+The GenBank file usually ends with .gb or sometimes .gbk. The GenBank
+format for protein has been renamed to GenPept. The GenBank (for
+nucleotide) and Genpept are essentially the same format. An example of
+a GenBank file can be seen here [1]_.
 
 Format Support
 --------------
@@ -39,47 +37,115 @@ Format Specification
 --------------------
 **State: Experimental as of 0.4.1.**
 
-The International Nucleotide Sequence Database Collaboration (INSDC)
-foundational initiative between the DDBJ, EMBL, and GenBank
-(http://www.insdc.org/). These organisations all use the
-same "Feature Table" layout in their plain text flat file formats.
-
-However, the header and sequence sections of an EMBL file are very
-different in layout to those produced by GenBank/DDBJ.
-
-Feature Table Documentation:
-http://www.insdc.org/files/feature_table.html
-ftp://ftp.ncbi.nih.gov/genbank/docs/FTv10_3.html
-
-The sequence in the ``'ORIGIN'`` section is always in lowercase for the
-GenBank files downloaded from NCBI. For the RNA molecules, ``'t'`` (thymine),
-instead of ``'u'`` (uracil) is used in the sequence. All GenBank writers
-follow these conventions while writing GenBank files.
-
-All the sections before ``'FEATURES'`` will be read into ``metadata`` of
-``Sequence`` or its sub-class. The header and its content of a section
-is stored as a pair of key and value in ``metadata``. For the ``'REFERENCE'``
+Sections before ``FEATURES``
+^^^^^^^^^^^^^^^^^^^^^^^^^^^^
+All the sections before ``FEATURES`` will be read into the attribute
+of ``metadata``. The header and its content of a section is stored as
+a pair of key and value in ``metadata``. For the ``REFERENCE``
 section, its value is stored as a list, as there are often multiple
 reference sections in one GenBank record.
 
-The information of the ``'FEATURES'`` is stored in both ``metadata`` and
-``positional_metadata`` of ``Sequence`` or its sub-class. For each feature,
-its location is stored as boolean column in ``positional_metadata``; other
-qualifiers are stored as a ``dict`` in the ``list`` of
-``metadata['FEATURES']``. In the ``dict`` of qualifiers, there are a few
-extra keys, which end with ``'_'``, including:
+``FEATURES`` section
+^^^^^^^^^^^^^^^^^^^^
+The International Nucleotide Sequence Database Collaboration (INSDC
+[2]_) is a joint effort among the DDBJ, EMBL, and GenBank. These
+organisations all use the same "Feature Table" layout in their plain
+text flat file formats, which are documented in detail [3]_. The
+feature keys and their qualifiers are also described in this webpage
+[4]_.
 
-    1. ``'index_'``: the column index to the ``positional_metadata``,
-where the location of the current feature is stored.
+The ``FEATURES`` section will be stored in ``interval_metadata`` of
+``Sequence`` or its sub-class. Each sub-section is stored as an
+``Interval`` object in ``interval_metadata``. Each ``Interval`` object
+has ``metadata`` keeping the information of this feature in the
+sub-section.
 
-    2. ``'left_partial_'``: whether the exact lower boundary point of the
-feature is unknown.
+To normalize the vocabulary between multiple formats (currently only
+the INSDC Feature Table and GFF3) to store metadata of interval
+features, we rename some terms in some formats to the same common name
+when parsing them into memory, as described in this table:
 
-    3. ``'right_partial_'``: whether the exact upper boundary point of the
-feature is unknown.
++-----------+-----------+-----------+---------+------------------------------+
+|INSDC      |GFF3       |Key stored |Value    |Description                   |
+|feature    |columns or |           |type     |                              |
+|table      |attributes |           |stored   |                              |
++===========+===========+===========+=========+==============================+
+|inference  |source     |source     |str      |the algorithm or experiment   |
+|           |(column 2) |           |         |used to generate this feature |
++-----------+-----------+-----------+---------+------------------------------+
+|feature key|type       |type       |str      |the type of the feature       |
+|           |(column 3) |           |         |                              |
++-----------+-----------+-----------+---------+------------------------------+
+|N/A        |score      |score      |float    |the score of the feature      |
+|           |(column 6) |           |         |                              |
++-----------+-----------+-----------+---------+------------------------------+
+|N/A        |strand     |strand     |str      |the strand of the feature. +  |
+|           |(column 7) |           |         |for positive strand, - for    |
+|           |           |           |         |minus strand, and . for       |
+|           |           |           |         |features that are not         |
+|           |           |           |         |stranded. In addition, ?  can |
+|           |           |           |         |be used for features whose    |
+|           |           |           |         |strandedness is relevant, but |
+|           |           |           |         |unknown.                      |
++-----------+-----------+-----------+---------+------------------------------+
+|codon_start|phase      |phase      |int      |the offset at which the first |
+|           |(column 8) |           |         |complete codon of a coding    |
+|           |           |           |         |feature can be found, relative|
+|           |           |           |         |to the first base of that     |
+|           |           |           |         |feature. It is 0, 1, or 2 in  |
+|           |           |           |         |GFF3 or 1, 2, or 3 in GenBank.|
+|           |           |           |         |The stored value is 0, 1, or  |
+|           |           |           |         |2, following in GFF3 format.  |
++-----------+-----------+-----------+---------+------------------------------+
+|db_xref    |Dbxref     |db_xref    |list of  |A database cross reference    |
+|           |           |           |str      |                              |
++-----------+-----------+-----------+---------+------------------------------+
+|N/A        |ID         |ID         |str      |feature ID                    |
++-----------+-----------+-----------+---------+------------------------------+
+|note       |Note       |note       |str      |any comment or additional     |
+|           |           |           |         |information                   |
++-----------+-----------+-----------+---------+------------------------------+
+|translation|N/A        |translation|str      |the protein sequence for CDS  |
+|           |           |           |         |features                      |
++-----------+-----------+-----------+---------+------------------------------+
+
+``Location`` string
++++++++++++++++++++
+There are 5 types of location descriptors defined in Feature
+Table. This explains how they will be parsed into the bounds of
+``Interval`` object (note it converts the 1-based coordinate to
+0-based):
+
+    1. a single base number. e.g. 67. This is parsed to ``(66, 67)``.
+
+    2. a site between two neighboring bases. e.g. 67^68. This
+       is parsed to ``(66, 67)``.
+
+    3. a single base from inside a range. e.g. 67.89. This is parsed to
+       ``(66, 89)``.
+
+    4. a pair of base numbers defining a sequence span. e.g. 67..89. This
+       is parsed to ``(66, 89)``.
+
+    5. a remote sequence identifier followed by a location descriptor
+       defined above. e.g. J00123.1:67..89. This will be discarded
+       because it is not on the current sequence. When it is combined
+       with local descriptor like J00123.1:67..89,200..209, the
+       local part will be kept to be ``(199, 209)``.
+
+
+.. note:: The Location string is fully stored in ``Interval.metadata``
+   with key ``__location``.  The key starting with ``__`` is "private"
+   and should be modified with care.
+
+
+``ORIGIN`` section
+^^^^^^^^^^^^^^^^^^
+The sequence in the ``ORIGIN`` section is always in lowercase for
+the GenBank files downloaded from NCBI. For the RNA molecules, ``t``
+(thymine), instead of ``u`` (uracil) is used in the sequence. All
+GenBank writers follow these conventions while writing GenBank files.
 
-    4. ``'type_'``: the molecular type of the feature. Its value is from the
-header of the feature.
 
 Format Parameters
 -----------------
@@ -90,12 +156,12 @@ The ``constructor`` parameter can be used with the ``Sequence`` generator
 to specify the in-memory type of each GenBank record that is parsed.
 ``constructor`` should be ``Sequence`` or a sub-class of ``Sequence``.
 It is also detected by the unit label on the LOCUS line. For example, if it
-is ``'bp'``, it will be read into ``DNA``; if it is ``'aa'``, it will be read
+is ``bp``, it will be read into ``DNA``; if it is ``aa``, it will be read
 into ``Protein``. Otherwise, it will be read into ``Sequence``. This default
 behavior is overridden by setting ``constructor``.
 
 ``lowercase`` is another parameter available for all GenBank readers.
-By default, it is set to ``True`` to read in the ``'ORIGIN'`` sequence
+By default, it is set to ``True`` to read in the ``ORIGIN`` sequence
 as lowercase letters. This parameter is passed to ``Sequence`` or
 its sub-class constructor.
 
@@ -108,57 +174,42 @@ Examples
 
 Reading and Writing GenBank Files
 ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
-Suppose we have the following GenBank file [example modified from [1_]::
+Suppose we have the following GenBank file example modified from [5]_:
 
-    LOCUS       3K1V_A       34 bp    RNA     linear   SYN 10-OCT-2012
-    DEFINITION  Chain A, Structure Of A Mutant Class-I Preq1.
-    ACCESSION   3K1V_A
-    VERSION     3K1V_A  GI:260656459
-    KEYWORDS    .
-    SOURCE      synthetic construct
-      ORGANISM  synthetic construct
-                other sequences; artificial sequences.
-    REFERENCE   1  (bases 1 to 34)
-      AUTHORS   Klein,D.J., Edwards,T.E. and Ferre-D'Amare,A.R.
-      TITLE     Cocrystal structure of a class I preQ1 riboswitch
-      JOURNAL   Nat. Struct. Mol. Biol. 16 (3), 343-344 (2009)
-       PUBMED   19234468
-    COMMENT     SEQRES.
-    FEATURES             Location/Qualifiers
-         source          1..34
-                         /organism="synthetic construct"
-                         /mol_type="other RNA"
-                         /db_xref="taxon:32630"
-    ORIGIN
-            1 agaggttcta gcacatccct ctataaaaaa ctaa
-    //
+>>> gb_str = '''
+... LOCUS       3K1V_A       34 bp    RNA     linear   SYN 10-OCT-2012
+... DEFINITION  Chain A, Structure Of A Mutant Class-I Preq1.
+... ACCESSION   3K1V_A
+... VERSION     3K1V_A  GI:260656459
+... KEYWORDS    .
+... SOURCE      synthetic construct
+...   ORGANISM  synthetic construct
+...             other sequences; artificial sequences.
+... REFERENCE   1  (bases 1 to 34)
+...   AUTHORS   Klein,D.J., Edwards,T.E. and Ferre-D'Amare,A.R.
+...   TITLE     Cocrystal structure of a class I preQ1 riboswitch
+...   JOURNAL   Nat. Struct. Mol. Biol. 16 (3), 343-344 (2009)
+...    PUBMED   19234468
+... COMMENT     SEQRES.
+... FEATURES             Location/Qualifiers
+...      source          1..34
+...                      /organism="synthetic construct"
+...                      /mol_type="other RNA"
+...                      /db_xref="taxon:32630"
+...      misc_binding    1..30
+...                      /note="Preq1 riboswitch"
+...                      /bound_moiety="preQ1"
+... ORIGIN
+...         1 agaggttcta gcacatccct ctataaaaaa ctaa
+... //
+... '''
 
->>> gb = ['LOCUS       3K1V_A     34 bp   RNA    linear   SYN 10-OCT-2012\\n',
-...       'DEFINITION  Chain A, Structure Of A Mutant Class-I Preq1.\\n',
-...       'ACCESSION   3K1V_A\\n',
-...       'VERSION     3K1V_A  GI:260656459\\n',
-...       'KEYWORDS    .\\n',
-...       'SOURCE      synthetic construct\\n',
-...       '  ORGANISM  synthetic construct\\n',
-...       '            other sequences; artificial sequences.\\n',
-...       'REFERENCE   1  (bases 1 to 34)\\n',
-...       "  AUTHORS   Klein,D.J., Edwards,T.E. and Ferre-D'Amare,A.R.\\n",
-...       '  TITLE     Cocrystal structure of a class I preQ1 riboswitch\\n',
-...       '  JOURNAL   Nat. Struct. Mol. Biol. 16 (3), 343-344 (2009)\\n',
-...       '   PUBMED   19234468\\n',
-...       'COMMENT     SEQRES.\\n',
-...       'FEATURES             Location/Qualifiers\\n',
-...       '     source          1..34\\n',
-...       '                     /organism="synthetic construct"\\n',
-...       '                     /mol_type="other RNA"\\n',
-...       '                     /db_xref="taxon:32630"\\n',
-...       'ORIGIN\\n',
-...       '        1 agaggttcta gcacatccct ctataaaaaa ctaa\\n',
-...       '//\\n']
 
 Now we can read it as ``DNA`` object:
 
+>>> import io
 >>> from skbio import DNA, RNA, Sequence
+>>> gb = io.StringIO(gb_str)
 >>> dna_seq = DNA.read(gb)
 >>> dna_seq
 DNA
@@ -167,28 +218,28 @@ Metadata:
     'ACCESSION': '3K1V_A'
     'COMMENT': 'SEQRES.'
     'DEFINITION': 'Chain A, Structure Of A Mutant Class-I Preq1.'
-    'FEATURES': <class 'list'>
     'KEYWORDS': '.'
     'LOCUS': <class 'dict'>
     'REFERENCE': <class 'list'>
     'SOURCE': <class 'dict'>
     'VERSION': '3K1V_A  GI:260656459'
-Positional metadata:
-    0: <dtype: bool>
+Interval metadata:
+    2 interval features
 Stats:
     length: 34
     has gaps: False
     has degenerates: False
-    has non-degenerates: True
+    has definites: True
     GC-content: 35.29%
 -----------------------------------------------------------------
 0 AGAGGTTCTA GCACATCCCT CTATAAAAAA CTAA
 
 
-Since this is a riboswitch molecule, we may want to read it as ``RNA``.
-As the GenBank file usually have ``'t'`` instead of ``'u'`` in the
-sequence, we can read it as ``RNA`` by converting ``'t'`` to ``'u'``:
+Since this is a riboswitch molecule, we may want to read it as
+``RNA``.  As the GenBank file usually have ``t`` instead of ``u`` in
+the sequence, we can read it as ``RNA`` by converting ``t`` to ``u``:
 
+>>> gb = io.StringIO(gb_str)
 >>> rna_seq = RNA.read(gb)
 >>> rna_seq
 RNA
@@ -197,19 +248,18 @@ Metadata:
     'ACCESSION': '3K1V_A'
     'COMMENT': 'SEQRES.'
     'DEFINITION': 'Chain A, Structure Of A Mutant Class-I Preq1.'
-    'FEATURES': <class 'list'>
     'KEYWORDS': '.'
     'LOCUS': <class 'dict'>
     'REFERENCE': <class 'list'>
     'SOURCE': <class 'dict'>
     'VERSION': '3K1V_A  GI:260656459'
-Positional metadata:
-    0: <dtype: bool>
+Interval metadata:
+    2 interval features
 Stats:
     length: 34
     has gaps: False
     has degenerates: False
-    has non-degenerates: True
+    has definites: True
     GC-content: 35.29%
 -----------------------------------------------------------------
 0 AGAGGUUCUA GCACAUCCCU CUAUAAAAAA CUAA
@@ -217,8 +267,7 @@ Stats:
 >>> rna_seq == dna_seq.transcribe()
 True
 
->>> from io import StringIO
->>> with StringIO() as fh:
+>>> with io.StringIO() as fh:
 ...     print(dna_seq.write(fh, format='genbank').getvalue())
 LOCUS       3K1V_A   34 bp   RNA   linear   SYN   10-OCT-2012
 DEFINITION  Chain A, Structure Of A Mutant Class-I Preq1.
@@ -235,10 +284,13 @@ REFERENCE   1  (bases 1 to 34)
   PUBMED    19234468
 COMMENT     SEQRES.
 FEATURES             Location/Qualifiers
-       source        1..34
+     source          1..34
                      /db_xref="taxon:32630"
                      /mol_type="other RNA"
                      /organism="synthetic construct"
+     misc_binding    1..30
+                     /bound_moiety="preQ1"
+                     /note="Preq1 riboswitch"
 ORIGIN
         1 agaggttcta gcacatccct ctataaaaaa ctaa
 //
@@ -246,7 +298,11 @@ ORIGIN
 
 References
 ----------
-.. [1_] http://www.ncbi.nlm.nih.gov/nuccore/3K1V_A
+.. [1] http://www.ncbi.nlm.nih.gov/Sitemap/samplerecord.html
+.. [2] http://www.insdc.org/
+.. [3] http://www.insdc.org/files/feature_table.html
+.. [4] http://www.ebi.ac.uk/ena/WebFeat/
+.. [5] http://www.ncbi.nlm.nih.gov/nuccore/3K1V_A
 
 """
 
@@ -258,14 +314,7 @@ References
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-from future.builtins import range, zip
-
 import re
-import numpy as np
-import pandas as pd
-from datetime import datetime
 from functools import partial
 
 from skbio.io import create_format, GenBankFormatError
@@ -273,12 +322,13 @@ from skbio.io.format._base import (
     _get_nth_sequence, _line_generator, _too_many_blanks)
 from skbio.util._misc import chunk_str
 from skbio.sequence import Sequence, DNA, RNA, Protein
+from skbio.io.format._sequence_feature_vocabulary import (
+    _yield_section, _parse_section_default, _serialize_section_default,
+    _parse_feature_table, _serialize_feature_table)
 
 
 genbank = create_format('genbank')
 
-# date format in locus line of genbank record
-_TIME_FORMAT = '%d-%b-%Y'
 # This list is ordered
 # used to read and write genbank file.
 _HEADERS = ['LOCUS',
@@ -371,7 +421,7 @@ def _protein_to_genbank(obj, fh):
 def _construct(record, constructor=None, **kwargs):
     '''Construct the object of Sequence, DNA, RNA, or Protein.
     '''
-    seq, md, pmd = record
+    seq, md, imd = record
     if 'lowercase' not in kwargs:
         kwargs['lowercase'] = True
     if constructor is None:
@@ -384,10 +434,10 @@ def _construct(record, constructor=None, **kwargs):
 
     if constructor == RNA:
         return DNA(
-            seq, metadata=md, positional_metadata=pmd, **kwargs).transcribe()
+            seq, metadata=md, interval_metadata=imd, **kwargs).transcribe()
     else:
         return constructor(
-            seq, metadata=md, positional_metadata=pmd, **kwargs)
+            seq, metadata=md, interval_metadata=imd, **kwargs)
 
 
 def _parse_genbanks(fh):
@@ -402,7 +452,7 @@ def _parse_genbanks(fh):
 
 def _parse_single_genbank(chunks):
     metadata = {}
-    positional_metadata = None
+    interval_metadata = None
     sequence = ''
     # each section starts with a HEADER without indent.
     section_splitter = _yield_section(
@@ -429,11 +479,10 @@ def _parse_single_genbank(chunks):
         elif header == 'ORIGIN':
             sequence = parsed
         elif header == 'FEATURES':
-            metadata[header] = parsed[0]
-            positional_metadata = pd.concat(parsed[1], axis=1)
+            interval_metadata = parsed
         else:
             metadata[header] = parsed
-    return sequence, metadata, positional_metadata
+    return sequence, metadata, interval_metadata
 
 
 def _serialize_single_genbank(obj, fh):
@@ -442,12 +491,18 @@ def _serialize_single_genbank(obj, fh):
     Always write it in NCBI canonical way:
     1. sequence in lowercase
     2. 'u' as 't' even in RNA molecules.
+
+    Parameters
+    ----------
+    obj : Sequence or its child class
+
     '''
+    # write out the headers
     md = obj.metadata
     for header in _HEADERS:
+        serializer = _SERIALIZER_TABLE.get(
+            header, _serialize_section_default)
         if header in md:
-            serializer = _SERIALIZER_TABLE.get(
-                header, _serialize_section_default)
             out = serializer(header, md[header])
             # test if 'out' is a iterator.
             # cf. Effective Python Item 17
@@ -456,6 +511,16 @@ def _serialize_single_genbank(obj, fh):
                     fh.write(s)
             else:
                 fh.write(out)
+        if header == 'FEATURES':
+            if obj.has_interval_metadata():
+                # magic number 21: the amount of indentation before
+                # feature table starts as defined by INSDC
+                indent = 21
+                fh.write('{header:<{indent}}Location/Qualifiers\n'.format(
+                    header=header, indent=indent))
+                for s in serializer(obj.interval_metadata._intervals, indent):
+                    fh.write(s)
+    # write out the sequence
     # always write RNA seq as DNA
     if isinstance(obj, RNA):
         obj = obj.reverse_transcribe()
@@ -510,12 +575,11 @@ def _parse_locus(lines):
             "Could not parse the LOCUS line:\n%s" % line)
 
     res['size'] = int(res['size'])
-    res['date'] = datetime.strptime(res['date'], _TIME_FORMAT)
     return res
 
 
 def _serialize_locus(header, obj, indent=12):
-    '''Serilize LOCUS line.
+    '''Serialize LOCUS line.
 
     Parameters
     ----------
@@ -523,8 +587,6 @@ def _serialize_locus(header, obj, indent=12):
     '''
     # use 'or' to convert None to ''
     kwargs = {k: v or '' for k, v in obj.items()}
-    # convert datetime to str
-    kwargs['date'] = kwargs['date'].strftime(_TIME_FORMAT).upper()
 
     return ('{header:<{indent}}{locus_name}   {size} {unit}'
             '   {mol_type}   {shape}   {division}   {date}\n').format(
@@ -543,7 +605,7 @@ def _parse_reference(lines):
         skip_blanks=True, strip=False)
     for section in section_splitter(lines):
         label, data = _parse_section_default(
-            section, join_delimitor=' ', return_label=True)
+            section, join_delimiter=' ', return_label=True)
         res[label] = data
     return res
 
@@ -605,175 +667,6 @@ def _serialize_source(header, obj, indent=12):
     return s
 
 
-def _parse_features(lines, length):
-    '''Parse FEATURES field.
-    '''
-    features = []
-    positional_metadata = []
-    # skip the 1st FEATURES line
-    if lines[0].startswith('FEATURES'):
-        lines = lines[1:]
-    # magic number 20: the lines following header of each feature
-    # are at least indented with 20 spaces.
-    feature_indent = ' ' * 20
-    section_splitter = _yield_section(
-        lambda x: not x.startswith(feature_indent),
-        skip_blanks=True, strip=False)
-    for i, section in enumerate(section_splitter(lines)):
-        # print(i) ; continue
-        feature, pmd = _parse_single_feature(section, length, i)
-        features.append(feature)
-        positional_metadata.append(pmd)
-    return features, positional_metadata
-
-
-def _serialize_features(header, obj, indent=21):
-    first = True
-    for feature in obj:
-        if first:
-            first = False
-            yield '{header:<{indent}}Location/Qualifiers\n{feat}'.format(
-                header=header, indent=indent,
-                feat=_serialize_single_feature(feature, indent))
-        else:
-            yield _serialize_single_feature(feature, indent)
-
-
-def _parse_single_feature(lines, length, index):
-    '''Parse a feature.
-
-    Returns
-    -------
-    tuple
-        Tuple of a dict of `metadata` and a pandas.Series of
-        `positional_metadata` for the feature.
-
-    '''
-    feature = {}
-    feature['index_'] = index
-    # each component of a feature starts with '/', except the 1st
-    # component of location.
-    section_splitter = _yield_section(
-        lambda x: x.startswith('/'), strip=True)
-    first = True
-    for section in section_splitter(lines):
-        if first:
-            # first section is the Location string
-            first = False
-            type, location = _parse_section_default(
-                section, join_delimitor='', return_label=True)
-            feature['type_'] = type
-            feature['location'] = location
-            loc, loc_pmd = _parse_loc_str(location, length)
-            feature.update(loc)
-        else:
-            # following sections are Qualifiers
-            k, v = _parse_section_default(
-                section, label_delimitor='=',
-                join_delimitor=' ', return_label=True)
-            k = k[1:]
-
-            # some Qualifiers can appear multiple times
-            if k in feature:
-                if not isinstance(feature[k], list):
-                    feature[k] = [feature[k]]
-                feature[k].append(v)
-            else:
-                feature[k] = v
-    return feature, loc_pmd
-
-
-def _serialize_single_feature(obj, indent=21):
-    padding = ' ' * 8
-    qualifiers = []
-    for k in sorted(obj):
-        if k.endswith('_') or k in ('location', 'type'):
-            continue
-        v = obj[k]
-        if isinstance(v, list):
-            for vi in v:
-                qualifiers.append(_serialize_qualifier(k, vi))
-        else:
-            qualifiers.append(_serialize_qualifier(k, v))
-
-    qualifiers = [' ' * indent + i for i in qualifiers]
-    return '{header:>{indent}}{loc}\n{qualifiers}\n'.format(
-        header=obj['type_'] + padding, loc=obj['location'],
-        indent=indent, qualifiers='\n'.join(qualifiers))
-
-
-def _serialize_qualifier(key, value):
-    '''Serialize a Qualifier in a feature.
-
-    Parameters
-    ----------
-    value : int, str
-    '''
-    # if value is empty
-    if not value:
-        return '/%s' % key
-
-    return '/{k}={v}'.format(k=key, v=value)
-
-
-def _parse_loc_str(loc_str, length):
-    '''Parse location string.
-
-    Warning: This converts coordinates to 0-based from 1-based as
-    in GenBank format.
-
-    The location descriptor can be one of the following:
-    (a) a single base number. e.g. 467
-    (b) a site between two indicated adjoining bases. e.g. 123^124
-    (c) a single base chosen from within a specified range of bases (not
-        allowed for new entries). e.g. 102.110
-    (d) the base numbers delimiting a sequence span. e.g.340..565
-    (e) a remote entry identifier followed by a local location
-        descriptor (i.e., a-d). e.g. J00194.1:100..202
-
-    TODO:
-    handle (b), (c), (e) cases correctly
-    '''
-    pmd = np.zeros(length, dtype=bool)
-    res = {'rc_': False,
-           'left_partial_': False,
-           'right_partial_': False}
-    items = re.split('[(),]+', loc_str)
-    operators = ['join', 'complement', 'order']
-    if 'complement' in items:
-        res['rc_'] = True
-    for i in items:
-        i = i.strip()
-        if i in operators or not i:
-            continue
-        elif ':' in i:  # (e)
-            index = []
-        elif '..' in i:  # (d)
-            beg, end = i.split('..')
-            if beg.startswith('<'):
-                beg = beg[1:]
-                res['left_partial_'] = True
-            if end.startswith('>'):
-                end = end[1:]
-                res['right_partial_'] = True
-            beg = int(beg)
-            end = int(end)
-            index = range(beg-1, end)
-        elif '.' in i:  # (c)
-            index = []
-        elif i.isdigit():  # (a)
-            index = int(i) - 1
-        elif '^' in i:  # (b)
-            index = []
-        else:
-            raise GenBankFormatError(
-                'Could not parse location string: "%s"' %
-                loc_str)
-        pmd[index] = True
-
-    return res, pd.Series(pmd)
-
-
 def _parse_origin(lines):
     '''Parse the ORIGIN section for sequence.
     '''
@@ -807,79 +700,11 @@ def _serialize_origin(seq, indent=9):
         yield s
 
 
-def _parse_section_default(
-        lines, label_delimitor=None, join_delimitor=' ', return_label=False):
-    '''Parse sections in default way.
-
-    Do 2 things:
-        1. split first line with label_delimitor for label
-        2. join all the lines into one str with join_delimitor.
-    '''
-    data = []
-    first = True
-    label = None
-    for line in lines:
-        if first:
-            items = line.split(label_delimitor, 1)
-
-            if len(items) == 2:
-                label, section = items
-            else:
-                label = items[0]
-                section = ""
-            data.append(section)
-            first = False
-        else:
-            data.append(line)
-    data = join_delimitor.join(i.strip() for i in data)
-    if return_label:
-        return label, data
-    else:
-        return data
-
-
-def _serialize_section_default(header, obj, indent=12):
-    return '{header:<{indent}}{obj}\n'.format(
-        header=header, obj=obj, indent=indent)
-
-
-def _yield_section(is_another_section, **kwargs):
-    '''Returns function that returns successive sections from file.
-
-    Parameters
-    ----------
-    is_another_section : callable
-        It takes a string as input and return a boolean indicating
-        a new section starts.
-    kwargs : dict, optional
-        Keyword arguments will be passed to `_line_generator`.
-
-    Returns
-    -------
-    function
-        A function accept a list of lines as input and return
-        a generator to yield section one by one.
-    '''
-    def parser(lines):
-        curr = []
-        for line in _line_generator(lines, **kwargs):
-            # if we find another, return the previous section
-            if is_another_section(line):
-                if curr:
-                    yield curr
-                    curr = []
-            curr.append(line)
-        # don't forget to return the last section in the file
-        if curr:
-            yield curr
-    return parser
-
-
 _PARSER_TABLE = {
     'LOCUS': _parse_locus,
     'SOURCE': _parse_source,
     'REFERENCE': _parse_reference,
-    'FEATURES': _parse_features,
+    'FEATURES': _parse_feature_table,
     'ORIGIN': _parse_origin}
 
 
@@ -887,4 +712,4 @@ _SERIALIZER_TABLE = {
     'LOCUS': _serialize_locus,
     'SOURCE': _serialize_source,
     'REFERENCE': _serialize_reference,
-    'FEATURES': _serialize_features}
+    'FEATURES': _serialize_feature_table}
