@@ -220,11 +220,6 @@ References
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import (absolute_import, division, print_function,
-                        unicode_literals)
-
-from future.builtins import zip, range
-
 from skbio.io import create_format, NewickFormatError
 from skbio.tree import TreeNode
 
