@@ -6,10 +6,7 @@
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
-from future.builtins import map, range, zip
-import six
-
+import copy
 import io
 import string
 from unittest import TestCase, main
@@ -25,24 +22,29 @@ from skbio.io.format.fasta import (
     _fasta_to_tabular_msa, _generator_to_fasta,
     _sequence_to_fasta, _dna_to_fasta, _rna_to_fasta, _protein_to_fasta,
     _tabular_msa_to_fasta)
-from skbio.sequence._iupac_sequence import IUPACSequence
+from skbio.sequence._grammared_sequence import GrammaredSequence
 from skbio.util import get_data_path
 from skbio.util._decorator import classproperty, overrides
 
 
-class CustomSequence(IUPACSequence):
+class CustomSequence(GrammaredSequence):
     @classproperty
-    @overrides(IUPACSequence)
+    @overrides(GrammaredSequence)
     def gap_chars(cls):
         return set('-.')
 
     @classproperty
-    @overrides(IUPACSequence)
-    def nondegenerate_chars(cls):
+    @overrides(GrammaredSequence)
+    def default_gap_char(cls):
+        return '-'
+
+    @classproperty
+    @overrides(GrammaredSequence)
+    def definite_chars(cls):
         return set(string.ascii_letters)
 
     @classproperty
-    @overrides(IUPACSequence)
+    @overrides(GrammaredSequence)
     def degenerate_map(cls):
         return {}
 
@@ -474,7 +476,7 @@ class ReaderTests(TestCase):
                 obs = list(_fasta_to_generator(fasta_fp, **kwargs))
                 self.assertEqual(len(obs), len(exp))
                 for o, e in zip(obs, exp):
-                    e = e.copy()
+                    e = copy.copy(e)
                     del e.positional_metadata['quality']
                     self.assertEqual(o, e)
 
@@ -488,7 +490,7 @@ class ReaderTests(TestCase):
 
     def test_fasta_to_generator_invalid_files(self):
         for fp, kwargs, error_type, error_msg_regex in self.invalid_fps:
-            with six.assertRaisesRegex(self, error_type, error_msg_regex):
+            with self.assertRaisesRegex(error_type, error_msg_regex):
                 list(_fasta_to_generator(fp, **kwargs))
 
     # light testing of fasta -> object readers to ensure interface is present
@@ -515,9 +517,9 @@ class ReaderTests(TestCase):
 
             # empty file
             empty_fp = get_data_path('empty')
-            with six.assertRaisesRegex(self, ValueError, '1st sequence'):
+            with self.assertRaisesRegex(ValueError, '1st sequence'):
                 reader_fn(empty_fp)
-            with six.assertRaisesRegex(self, ValueError, '1st sequence'):
+            with self.assertRaisesRegex(ValueError, '1st sequence'):
                 reader_fn(empty_fp, qual=empty_fp)
 
             # the sequences in the following files don't necessarily make sense
@@ -607,19 +609,17 @@ class ReaderTests(TestCase):
                     self.assertEqual(obs, exp)
 
                 # seq_num too large
-                with six.assertRaisesRegex(self, ValueError, '8th sequence'):
+                with self.assertRaisesRegex(ValueError, '8th sequence'):
                     reader_fn(fasta_fp, seq_num=8)
                 for qual_fp in qual_fps:
-                    with six.assertRaisesRegex(self, ValueError,
-                                               '8th sequence'):
+                    with self.assertRaisesRegex(ValueError, '8th sequence'):
                         reader_fn(fasta_fp, seq_num=8, qual=qual_fp)
 
                 # seq_num too small
-                with six.assertRaisesRegex(self, ValueError, '`seq_num`=0'):
+                with self.assertRaisesRegex(ValueError, '`seq_num`=0'):
                     reader_fn(fasta_fp, seq_num=0)
                 for qual_fp in qual_fps:
-                    with six.assertRaisesRegex(self, ValueError,
-                                               '`seq_num`=0'):
+                    with self.assertRaisesRegex(ValueError, '`seq_num`=0'):
                         reader_fn(fasta_fp, seq_num=0, qual=qual_fp)
 
     def test_fasta_to_tabular_msa(self):
@@ -641,7 +641,7 @@ class ReaderTests(TestCase):
 
                 self.assertEqual(len(obs), len(exp))
                 for o, e in zip(obs, exp):
-                    e = e.copy()
+                    e = copy.copy(e)
                     del e.positional_metadata['quality']
                     self.assertEqual(o, e)
 
@@ -651,7 +651,7 @@ class ReaderTests(TestCase):
                     self.assertEqual(obs, exp)
 
     def test_fasta_to_tabular_msa_no_constructor(self):
-        with six.assertRaisesRegex(self, ValueError, '`constructor`'):
+        with self.assertRaisesRegex(ValueError, '`constructor`'):
             _fasta_to_tabular_msa(get_data_path('fasta_single_seq'))
 
 
@@ -709,8 +709,7 @@ class WriterTests(TestCase):
         self.msa = TabularMSA(seqs)
 
         def empty_gen():
-            raise StopIteration()
-            yield
+            yield from ()
 
         def single_seq_gen():
             yield self.bio_seq1
@@ -736,19 +735,17 @@ class WriterTests(TestCase):
         # including exercising the different splitting algorithms used for
         # sequence data vs. quality scores
         def multi_seq_gen():
-            for seq in (self.bio_seq1, self.bio_seq2, self.bio_seq3,
-                        self.dna_seq, self.rna_seq, self.prot_seq):
-                yield seq
+            yield from (self.bio_seq1, self.bio_seq2, self.bio_seq3,
+                        self.dna_seq, self.rna_seq, self.prot_seq)
 
         # can be serialized if no qual file is provided, else it should raise
         # an error because one seq has qual scores and the other doesn't
         def mixed_qual_score_gen():
-            missing_qual_seq = DNA(
-                'AAAAT', metadata={'id': 'da,dadadada',
-                                   'description': '10 hours'},
-                lowercase='introns')
-            for seq in self.bio_seq1, missing_qual_seq:
-                yield seq
+            yield self.bio_seq1
+            yield DNA('AAAAT',
+                      metadata={'id': 'da,dadadada',
+                                'description': '10 hours'},
+                      lowercase='introns')
 
         self.mixed_qual_score_gen = mixed_qual_score_gen()
 
@@ -807,8 +804,7 @@ class WriterTests(TestCase):
         ]))
 
         def blank_seq_gen():
-            for seq in self.bio_seq1, Sequence(''):
-                yield seq
+            yield from (self.bio_seq1, Sequence(''))
 
         # generators or parameter combos that cannot be written in fasta
         # format, paired with kwargs (if any), error type, and expected error
@@ -876,7 +872,7 @@ class WriterTests(TestCase):
     def test_generator_to_fasta_invalid_input(self):
         for obj, kwargs, error_type, error_msg_regexp in self.invalid_objs:
             fh = io.StringIO()
-            with six.assertRaisesRegex(self, error_type, error_msg_regexp):
+            with self.assertRaisesRegex(error_type, error_msg_regexp):
                 _generator_to_fasta(obj, fh, **kwargs)
             fh.close()
 
