@@ -6,11 +6,9 @@
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
-from six.moves import zip_longest
-
 from io import StringIO
 import io
+import itertools
 import os
 import unittest
 import warnings
@@ -28,7 +26,7 @@ from skbio.util._exception import TestingUtilError
 from skbio import DNA, read, write
 
 
-class TestClass(object):
+class TestClass:
     def __init__(self, l):
         self.list = l
 
@@ -556,7 +554,7 @@ class TestSniff(RegistryTest):
             return
 
     def test_no_matches(self):
-        fh = StringIO(u"no matches here")
+        fh = StringIO("no matches here")
         with self.assertRaises(UnrecognizedFormatError) as cm:
             self.registry.sniff(fh)
         self.assertTrue(str(fh) in str(cm.exception))
@@ -564,11 +562,11 @@ class TestSniff(RegistryTest):
         fh.close()
 
     def test_one_match(self):
-        fh = StringIO(u"contains a 3")
+        fh = StringIO("contains a 3")
         self.assertEqual('format3', self.registry.sniff(fh)[0])
 
     def test_many_matches(self):
-        fh = StringIO(u"1234 will match all")
+        fh = StringIO("1234 will match all")
         with self.assertRaises(UnrecognizedFormatError) as cm:
             self.registry.sniff(fh)
         self.assertTrue("format1" in str(cm.exception))
@@ -590,18 +588,17 @@ class TestSniff(RegistryTest):
         fmt, _ = self.registry.sniff(fp, encoding='big5')
         self.assertEqual(fmt, 'formatx')
 
-    def test_that_newline_is_used(self):
+    def test_passing_newline_raises_error(self):
         formatx = self.registry.create_format('formatx')
 
         fp = get_data_path('real_file')
 
         @formatx.sniffer()
         def sniffer(fh):
-            self.assertEqual(fh.readlines(), ['a\nb\nc\nd\ne\n'])
             return True, {}
 
-        fmt, _ = self.registry.sniff(fp, newline='\r')
-        self.assertEqual(fmt, 'formatx')
+        with self.assertRaisesRegex(TypeError, '`newline`'):
+            self.registry.sniff(fp, newline='\r')
 
     def test_non_default_encoding(self):
         big5_format = self.registry.create_format('big5_format',
@@ -627,17 +624,12 @@ class TestSniff(RegistryTest):
 
         @formatx.sniffer()
         def sniffer(fh):
-            self.assertEqual(fh.readlines(), self._expected_lines)
+            self.assertEqual(fh.readlines(), ['a\nb\nc\nd\ne\n'])
             return True, {}
 
-        self._expected_lines = ['a\nb\nc\nd\ne\n']
         fmt, _ = self.registry.sniff(fp)
         self.assertEqual(fmt, 'formatx')
 
-        self._expected_lines = ['a\n', 'b\n', 'c\n', 'd\n', 'e\n']
-        fmt, _ = self.registry.sniff(fp, newline=None)
-        self.assertEqual(fmt, 'formatx')
-
     def test_position_not_mutated_real_file(self):
         formatx = self.registry.create_format('formatx')
 
@@ -658,7 +650,7 @@ class TestSniff(RegistryTest):
         def sniffer(fh):
             return True, {}
 
-        fh = StringIO(u'a\nb\nc\nd\n')
+        fh = StringIO('a\nb\nc\nd\n')
         fh.seek(2)
         self.registry.sniff(fh)
         self.assertEqual('b\n', fh.readline())
@@ -734,7 +726,7 @@ class TestSniff(RegistryTest):
         self.assertEqual(fmt, 'binf')
 
         with self.assertRaises(ValueError):
-            self.registry.sniff([u'some content\n'], encoding='binary')
+            self.registry.sniff(['some content\n'], encoding='binary')
 
         with self.assertRaises(ValueError):
             binf_sniffer(self.fp1, encoding=None)
@@ -766,7 +758,7 @@ class TestSniff(RegistryTest):
         self.assertTrue(self._check_textf)
 
     def test_sniff_gzip(self):
-        expected = u"This is some content\nIt occurs on more than one line\n"
+        expected = "This is some content\nIt occurs on more than one line\n"
 
         formata = self.registry.create_format('formata', encoding='binary')
         formatb = self.registry.create_format('formatb')
@@ -815,7 +807,7 @@ class TestSniff(RegistryTest):
         self._check_binf = False
         self._check_textf = False
 
-        fmt, _ = self.registry.sniff([u'text'])
+        fmt, _ = self.registry.sniff(['text'])
         self.assertEqual(fmt, 'textf')
 
         self.assertFalse(self._check_binf)
@@ -865,7 +857,7 @@ class TestRead(RegistryTest):
     def test_format_is_none(self):
         format1 = self.registry.create_format('format1')
 
-        fh = StringIO(u'1\n2\n3\n4')
+        fh = StringIO('1\n2\n3\n4')
 
         @format1.sniffer()
         def sniffer(fh):
@@ -880,16 +872,30 @@ class TestRead(RegistryTest):
         self.assertEqual(TestClass([1, 2, 3, 4]), instance)
         fh.close()
 
+    def test_into_is_none_and_no_generator_reader(self):
+        format1 = self.registry.create_format('format1')
+
+        fh = StringIO('1\n2\n3\n4')
+
+        @format1.reader(TestClass)
+        def reader(fh):
+            self.assertIsInstance(fh, io.TextIOBase)
+            return
+
+        with self.assertRaisesRegex(UnrecognizedFormatError,
+                                    "Cannot read 'format1'.*Possible.*include"
+                                    ": TestClass"):
+            self.registry.read(fh, format='format1')
+
     def test_into_is_none(self):
         format1 = self.registry.create_format('format1')
 
-        fh = StringIO(u'1\n2\n3\n4')
+        fh = StringIO('1\n2\n3\n4')
 
         @format1.reader(None)
         def reader(fh):
             self.assertIsInstance(fh, io.TextIOBase)
-            for value in [int(x) for x in fh.read().split('\n')]:
-                yield value
+            yield from [int(x) for x in fh.read().split('\n')]
 
         generator = self.registry.read(fh, format='format1')
         self.assertIsInstance(generator, types.GeneratorType)
@@ -914,11 +920,10 @@ class TestRead(RegistryTest):
         @format1.reader(None)
         def reader(fh):
             self._test_fh = fh
-            for value in [int(x) for x in fh.read().split('\n')]:
-                yield value
+            yield from [int(x) for x in fh.read().split('\n')]
 
         generator = self.registry.read(fp, format='format1')
-        for a, b in zip_longest(generator, [1, 2, 3, 4]):
+        for a, b in itertools.zip_longest(generator, [1, 2, 3, 4]):
             self.assertEqual(a, b)
         self.assertTrue(self._test_fh.closed)
 
@@ -939,7 +944,7 @@ class TestRead(RegistryTest):
     def test_reader_exists_with_verify_true(self):
         format1 = self.registry.create_format('format1')
 
-        fh = StringIO(u'1\n2\n3\n4')
+        fh = StringIO('1\n2\n3\n4')
 
         @format1.sniffer()
         def sniffer(fh):
@@ -969,7 +974,7 @@ class TestRead(RegistryTest):
     def test_warning_raised(self):
         format1 = self.registry.create_format('format1')
 
-        fh = StringIO(u'1\n2\n3\n4')
+        fh = StringIO('1\n2\n3\n4')
 
         @format1.sniffer()
         def sniffer(fh):
@@ -1003,7 +1008,7 @@ class TestRead(RegistryTest):
     def test_reader_exists_with_verify_false(self):
         format1 = self.registry.create_format('format1')
 
-        fh = StringIO(u'1\n2\n3\n4')
+        fh = StringIO('1\n2\n3\n4')
 
         @format1.sniffer()
         def sniffer(fh):
@@ -1069,16 +1074,16 @@ class TestRead(RegistryTest):
             self.assertEqual(kwargs['arg3'], [1])
             return
 
-        self.registry.read(StringIO(u'notempty'), into=TestClass, arg3=[1])
+        self.registry.read(StringIO('notempty'), into=TestClass, arg3=[1])
 
         with warnings.catch_warnings(record=True):
             warnings.simplefilter("error")
             # Should raise no warning and thus no error.
-            self.registry.read(StringIO(u'notempty'), into=TestClass, arg3=[1],
+            self.registry.read(StringIO('notempty'), into=TestClass, arg3=[1],
                                override=30)
             # Should raise a warning and thus an error.
             with self.assertRaises(ArgumentOverrideWarning):
-                self.registry.read(StringIO(u'notempty'), into=TestClass,
+                self.registry.read(StringIO('notempty'), into=TestClass,
                                    arg3=[1], override=100)
 
     def test_that_encoding_is_used(self):
@@ -1088,7 +1093,7 @@ class TestRead(RegistryTest):
 
         @format1.sniffer()
         def sniffer(fh):
-            return u'\u4f60' in fh.readline(), {}
+            return '\u4f60' in fh.readline(), {}
 
         @format1.reader(TestClass)
         def reader(fh):
@@ -1102,11 +1107,11 @@ class TestRead(RegistryTest):
 
         self._expected_enc = 'big5'
         instance = self.registry.read(fp, into=TestClass, encoding='big5')
-        self.assertEqual(TestClass([u'\u4f60\u597d\n']), instance)
+        self.assertEqual(TestClass(['\u4f60\u597d\n']), instance)
 
         self._expected_enc = 'big5'
         gen = self.registry.read(fp, format='format1', encoding='big5')
-        self.assertEqual(TestClass([u'\u4f60\u597d\n']), next(gen))
+        self.assertEqual(TestClass(['\u4f60\u597d\n']), next(gen))
 
     def test_non_default_encoding(self):
         format1 = self.registry.create_format('format1', encoding='big5')
@@ -1129,10 +1134,10 @@ class TestRead(RegistryTest):
 
         self._expected_enc = 'big5'
         instance = self.registry.read(fp, into=TestClass)
-        self.assertEqual(TestClass([u'\u4f60\u597d\n']), instance)
+        self.assertEqual(TestClass(['\u4f60\u597d\n']), instance)
 
         gen = self.registry.read(fp, format='format1')
-        self.assertEqual(TestClass([u'\u4f60\u597d\n']), next(gen))
+        self.assertEqual(TestClass(['\u4f60\u597d\n']), next(gen))
         gen.close()
 
         self._expected_enc = 'utf8'
@@ -1142,7 +1147,7 @@ class TestRead(RegistryTest):
         with self.assertRaises(UnicodeDecodeError):
             self.registry.read(fp, format='format1', encoding='utf8')
 
-    def test_that_newline_is_used(self):
+    def test_passing_newline_raises_error(self):
         formatx = self.registry.create_format('formatx')
 
         fp = get_data_path('real_file')
@@ -1159,12 +1164,11 @@ class TestRead(RegistryTest):
         def reader_gen(fh):
             yield TestClass(fh.readlines())
 
-        instance = self.registry.read(fp, into=TestClass, newline='\r')
-        self.assertEqual(instance, TestClass(['a\nb\nc\nd\ne\n']))
+        with self.assertRaisesRegex(TypeError, '`newline`'):
+            self.registry.read(fp, into=TestClass, newline='\r')
 
-        gen = self.registry.read(fp, format='formatx', newline='\r')
-        self.assertEqual(next(gen), TestClass(['a\nb\nc\nd\ne\n']))
-        gen.close()
+        with self.assertRaisesRegex(TypeError, '`newline`'):
+            self.registry.read(fp, format='formatx', newline='\r')
 
     def test_non_default_newline(self):
         formatx = self.registry.create_format('formatx', newline='\r')
@@ -1190,21 +1194,12 @@ class TestRead(RegistryTest):
         self.assertEqual(next(gen), TestClass(['a\nb\nc\nd\ne\n']))
         gen.close()
 
-        instance = self.registry.read(fp, into=TestClass, newline=None)
-        self.assertEqual(instance, TestClass(['a\n', 'b\n', 'c\n', 'd\n',
-                                              'e\n']))
-
-        gen = self.registry.read(fp, format='formatx', newline=None)
-        self.assertEqual(next(gen), TestClass(['a\n', 'b\n', 'c\n', 'd\n',
-                                               'e\n']))
-        gen.close()
-
     def test_file_sentinel_many(self):
         format1 = self.registry.create_format('format1')
 
         extra = get_data_path('real_file')
         extra_2 = get_data_path('real_file_2')
-        fh = StringIO(u'1\n2\n3\n4')
+        fh = StringIO('1\n2\n3\n4')
 
         @format1.sniffer()
         def sniffer(fh):
@@ -1225,7 +1220,7 @@ class TestRead(RegistryTest):
     def test_file_sentinel_converted_to_none(self):
         format1 = self.registry.create_format('format1')
 
-        fh = StringIO(u'1\n2\n3\n4')
+        fh = StringIO('1\n2\n3\n4')
 
         @format1.sniffer()
         def sniffer(fh):
@@ -1245,7 +1240,7 @@ class TestRead(RegistryTest):
     def test_file_sentinel_pass_none(self):
         format1 = self.registry.create_format('format1')
 
-        fh = StringIO(u'1\n2\n3\n4')
+        fh = StringIO('1\n2\n3\n4')
 
         @format1.sniffer()
         def sniffer(fh):
@@ -1268,7 +1263,7 @@ class TestRead(RegistryTest):
 
         extra = get_data_path('real_file')
         extra_2 = get_data_path('real_file_2')
-        fh = StringIO(u'1\n2\n3\n4')
+        fh = StringIO('1\n2\n3\n4')
 
         @format1.sniffer()
         def sniffer(fh):
@@ -1289,7 +1284,7 @@ class TestRead(RegistryTest):
     def test_file_sentinel_converted_to_none_generator(self):
         format1 = self.registry.create_format('format1')
 
-        fh = StringIO(u'1\n2\n3\n4')
+        fh = StringIO('1\n2\n3\n4')
 
         @format1.sniffer()
         def sniffer(fh):
@@ -1309,7 +1304,7 @@ class TestRead(RegistryTest):
     def test_file_sentinel_pass_none_generator(self):
         format1 = self.registry.create_format('format1')
 
-        fh = StringIO(u'1\n2\n3\n4')
+        fh = StringIO('1\n2\n3\n4')
 
         @format1.sniffer()
         def sniffer(fh):
@@ -1369,23 +1364,23 @@ class TestRead(RegistryTest):
         gen.close()
 
         with self.assertRaises(ValueError):
-            self.registry.read([u'some content\n'], encoding='binary',
+            self.registry.read(['some content\n'], encoding='binary',
                                into=TestClass)
 
         with self.assertRaises(ValueError):
-            self.registry.read([u'some content\n'], format='textf',
+            self.registry.read(['some content\n'], format='textf',
                                encoding='binary', into=TestClass)
 
         with self.assertRaises(ValueError):
-            self.registry.read([u'some content\n'], format='textf',
+            self.registry.read(['some content\n'], format='textf',
                                encoding='binary', verify=False, into=TestClass)
 
         with self.assertRaises(ValueError):
-            self.registry.read([u'some content\n'], format='textf',
+            self.registry.read(['some content\n'], format='textf',
                                encoding='binary')
 
         with self.assertRaises(ValueError):
-            self.registry.read([u'some content\n'], format='textf',
+            self.registry.read(['some content\n'], format='textf',
                                encoding='binary', verify=False)
 
         with self.assertRaises(ValueError):
@@ -1449,6 +1444,23 @@ class TestRead(RegistryTest):
         self.assertEqual(next(gen), TestClass(['woo']))
         gen.close()
 
+    def test_read_empty_file_gen_with_format(self):
+        format1 = self.registry.create_format('format1')
+
+        @format1.sniffer()
+        def sniffer(fh):
+            return True, {}
+
+        @format1.reader(None)
+        def reader1(fh):
+            return
+            yield
+
+        with io.StringIO("") as fh:
+            gen = self.registry.read(fh, format='format1')
+
+        self.assertEqual(list(gen), [])
+
 
 class TestWrite(RegistryTest):
     def test_writer_does_not_exist(self):
@@ -1469,7 +1481,7 @@ class TestWrite(RegistryTest):
         @format1.writer(TestClass)
         def writer(obj, fh):
             self.assertIsInstance(fh, io.TextIOBase)
-            fh.write(u'\n'.join(obj.list))
+            fh.write('\n'.join(obj.list))
 
         self.registry.write(obj, format='format1', into=fh)
         fh.seek(0)
@@ -1485,12 +1497,12 @@ class TestWrite(RegistryTest):
         @format1.writer(TestClass)
         def writer(obj, fh):
             self.assertIsInstance(fh, io.TextIOBase)
-            fh.write(u'\n'.join(obj.list))
+            fh.write('\n'.join(obj.list))
 
         self.registry.write(obj, format='format1', into=fp)
 
         with io.open(fp) as fh:
-            self.assertEqual(u"1\n2\n3\n4", fh.read())
+            self.assertEqual("1\n2\n3\n4", fh.read())
 
     def test_writer_passed_kwargs(self):
         format1 = self.registry.create_format('format1')
@@ -1510,12 +1522,12 @@ class TestWrite(RegistryTest):
     def test_that_encoding_is_used(self):
         format1 = self.registry.create_format('format1')
 
-        obj = TestClass([u'\u4f60\u597d\n'])  # Ni Hau
+        obj = TestClass(['\u4f60\u597d\n'])  # Ni Hau
         fp = self.fp1
 
         @format1.writer(TestClass)
         def writer(obj, fh):
-            fh.write(u''.join(obj.list))
+            fh.write(''.join(obj.list))
             self.assertEqual(self._expected_encoding, fh.encoding)
 
         self._expected_encoding = 'big5'
@@ -1528,12 +1540,12 @@ class TestWrite(RegistryTest):
     def test_non_default_encoding(self):
         format1 = self.registry.create_format('format1', encoding='big5')
 
-        obj = TestClass([u'\u4f60\u597d\n'])  # Ni Hau
+        obj = TestClass(['\u4f60\u597d\n'])  # Ni Hau
         fp = self.fp1
 
         @format1.writer(TestClass)
         def writer(obj, fh):
-            fh.write(u''.join(obj.list))
+            fh.write(''.join(obj.list))
             self.assertEqual(self._expected_encoding, fh.encoding)
 
         self._expected_encoding = 'big5'
@@ -1551,12 +1563,12 @@ class TestWrite(RegistryTest):
     def test_that_newline_is_used(self):
         format1 = self.registry.create_format('format1')
 
-        obj = TestClass([u'a\n', u'b\n', u'c\n'])
+        obj = TestClass(['a\n', 'b\n', 'c\n'])
         fp = self.fp1
 
         @format1.writer(TestClass)
         def writer(obj, fh):
-            fh.write(u''.join(obj.list))
+            fh.write(''.join(obj.list))
 
         self.registry.write(obj, format='format1', into=fp, newline='\r')
 
@@ -1566,12 +1578,12 @@ class TestWrite(RegistryTest):
     def test_non_default_newline(self):
         format1 = self.registry.create_format('format1', newline='\r')
 
-        obj = TestClass([u'a\n', u'b\n', u'c\n'])
+        obj = TestClass(['a\n', 'b\n', 'c\n'])
         fp = self.fp1
 
         @format1.writer(TestClass)
         def writer(obj, fh):
-            fh.write(u''.join(obj.list))
+            fh.write(''.join(obj.list))
 
         self.registry.write(obj, format='format1', into=fp)
 
@@ -1590,8 +1602,8 @@ class TestWrite(RegistryTest):
 
         @format1.writer(TestClass)
         def writer(obj, fh, extra=FileSentinel, other=2, extra_2=FileSentinel):
-            extra.write(u'oh yeah...')
-            extra_2.write(u'oh no...')
+            extra.write('oh yeah...')
+            extra_2.write('oh no...')
 
         self.registry.write(TestClass([]), format='format1', into=fh,
                             extra=self.fp1, extra_2=self.fp2)
@@ -1671,7 +1683,7 @@ class TestWrite(RegistryTest):
     def test_io_kwargs_passed(self):
         format1 = self.registry.create_format('format1', encoding='ascii')
 
-        obj = TestClass([u'a\n', u'b\n', u'c\n'])
+        obj = TestClass(['a\n', 'b\n', 'c\n'])
         fp = self.fp1
         f = io.BytesIO()
 
@@ -1699,13 +1711,13 @@ class TestMonkeyPatch(RegistryTest):
     def setUp(self):
         super(TestMonkeyPatch, self).setUp()
 
-        class UnassumingClass(object):
+        class UnassumingClass:
             pass
 
-        class ClassWithDefault(object):
+        class ClassWithDefault:
             default_write_format = 'favfmt'
 
-        class NoMonkeySee(object):
+        class NoMonkeySee:
             pass
 
         self.unassuming_class = UnassumingClass
@@ -1872,7 +1884,7 @@ class TestMonkeyPatch(RegistryTest):
             self.was_called = True
 
         self.registry.monkey_patch()
-        fh = StringIO(u'notempty')
+        fh = StringIO('notempty')
         self.class_with_default.read(fh, a='a', b=123)
 
         self.assertTrue(self.was_called)
@@ -1899,26 +1911,26 @@ class TestMonkeyPatch(RegistryTest):
 class TestModuleFunctions(unittest.TestCase):
 
     def test_sniff_matches(self):
-        exp = io_registry.sniff([u'(a, b);'])
-        result = sniff([u'(a, b);'])
+        exp = io_registry.sniff(['(a, b);'])
+        result = sniff(['(a, b);'])
         self.assertEqual(exp, result)
         self.assertEqual('newick', exp[0])
         self.assertEqual({}, exp[1])
 
     def test_read_matches(self):
-        input = [u'>\n', u'ACGT\n']
+        input = ['>\n', 'ACGT\n']
         exp = io_registry.read(input, into=DNA)
         result = read(input, into=DNA)
         self.assertEqual(exp, result)
-        self.assertEqual(exp, DNA('ACGT', metadata={u'id': u'',
-                                                    u'description': u''}))
+        self.assertEqual(exp, DNA('ACGT', metadata={'id': '',
+                                                    'description': ''}))
 
     def test_write_matches(self):
         input = DNA('ACGT')
         exp = io_registry.write(input, format='fasta', into=[])
         result = write(input, format='fasta', into=[])
         self.assertEqual(exp, result)
-        self.assertEqual(exp, [u'>\n', u'ACGT\n'])
+        self.assertEqual(exp, ['>\n', 'ACGT\n'])
 
     def test_create_format_matches(self):
         with self.assertRaises(DuplicateRegistrationError):
