@@ -6,16 +6,15 @@
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
-
 import numpy as np
 
 from skbio.util._decorator import classproperty, overrides
 from skbio.util._decorator import stable
-from ._iupac_sequence import IUPACSequence, _motifs as parent_motifs
+from ._grammared_sequence import (GrammaredSequence, DisableSubclassingMeta,
+                                  _motifs as parent_motifs)
 
 
-class Protein(IUPACSequence):
+class Protein(GrammaredSequence, metaclass=DisableSubclassingMeta):
     """Store protein sequence data and optional associated metadata.
 
     Only characters in the IUPAC protein character set [1]_ are supported.
@@ -30,6 +29,9 @@ class Protein(IUPACSequence):
         Arbitrary per-character metadata. For example, quality data from
         sequencing reads. Must be able to be passed directly to the Pandas
         DataFrame constructor.
+    interval_metadata : IntervalMetadata
+        Arbitrary interval metadata which applies to intervals within
+        a sequence to store interval features (such as protein domains).
     lowercase : bool or str, optional
         If ``True``, lowercase sequence characters will be converted to
         uppercase characters in order to be valid IUPAC Protein characters. If
@@ -52,15 +54,27 @@ class Protein(IUPACSequence):
     ----------
     values
     metadata
+    interval_metadata
     positional_metadata
     alphabet
     gap_chars
     default_gap_char
     stop_chars
-    nondegenerate_chars
+    definite_chars
     degenerate_chars
     degenerate_map
 
+    See Also
+    --------
+    GrammaredSequence
+
+    Notes
+    -----
+    Subclassing is disabled for Protein, because subclassing makes
+    it possible to change the alphabet, and certain methods rely on the
+    IUPAC alphabet. If a custom sequence alphabet is needed, inherit directly
+    from ``GrammaredSequence``.
+
     References
     ----------
     .. [1] Nomenclature for incompletely specified bases in nucleic acid
@@ -73,28 +87,28 @@ class Protein(IUPACSequence):
     >>> from skbio import Protein
     >>> Protein('PAW')
     Protein
-    -----------------------------
+    --------------------------
     Stats:
         length: 3
         has gaps: False
         has degenerates: False
-        has non-degenerates: True
+        has definites: True
         has stops: False
-    -----------------------------
+    --------------------------
     0 PAW
 
     Convert lowercase characters to uppercase:
 
     >>> Protein('paW', lowercase=True)
     Protein
-    -----------------------------
+    --------------------------
     Stats:
         length: 3
         has gaps: False
         has degenerates: False
-        has non-degenerates: True
+        has definites: True
         has stops: False
-    -----------------------------
+    --------------------------
     0 PAW
 
     """
@@ -108,20 +122,17 @@ class Protein(IUPACSequence):
         return cls.__stop_codes
 
     @classproperty
-    @stable(as_of="0.4.0")
-    @overrides(IUPACSequence)
+    @overrides(GrammaredSequence)
     def alphabet(cls):
         return super(Protein, cls).alphabet | cls.stop_chars
 
     @classproperty
-    @stable(as_of="0.4.0")
-    @overrides(IUPACSequence)
-    def nondegenerate_chars(cls):
+    @overrides(GrammaredSequence)
+    def definite_chars(cls):
         return set("ACDEFGHIKLMNPQRSTVWY")
 
     @classproperty
-    @stable(as_of="0.4.0")
-    @overrides(IUPACSequence)
+    @overrides(GrammaredSequence)
     def degenerate_map(cls):
         return {
             "B": set("DN"), "Z": set("EQ"),
@@ -141,6 +152,16 @@ class Protein(IUPACSequence):
         """
         return set('*')
 
+    @classproperty
+    @overrides(GrammaredSequence)
+    def gap_chars(cls):
+        return set('-.')
+
+    @classproperty
+    @overrides(GrammaredSequence)
+    def default_gap_char(cls):
+        return '-'
+
     @property
     def _motifs(self):
         return _motifs
@@ -195,7 +216,7 @@ class Protein(IUPACSequence):
         """
         return bool(self.stops().any())
 
-    @overrides(IUPACSequence)
+    @overrides(GrammaredSequence)
     def _repr_stats(self):
         """Define custom statistics to display in the sequence's repr."""
         stats = super(Protein, self)._repr_stats()
