@@ -6,29 +6,26 @@
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
-from future.builtins import range, zip
-from future.utils import viewitems
-import six
-
 import re
 import collections
 import numbers
 from contextlib import contextmanager
 
 import numpy as np
-from scipy.spatial.distance import hamming
-
 import pandas as pd
 
-from skbio._base import SkbioObject, MetadataMixin, PositionalMetadataMixin
+import skbio.sequence.distance
+from skbio._base import SkbioObject
+from skbio.metadata._mixin import (MetadataMixin, PositionalMetadataMixin,
+                                   IntervalMetadataMixin)
+from skbio.metadata import IntervalMetadata
 from skbio.sequence._repr import _SequenceReprBuilder
-from skbio.util._decorator import (stable, experimental, deprecated,
-                                   classonlymethod, overrides)
+from skbio.util._decorator import (stable, experimental, classonlymethod,
+                                   overrides)
 
 
-class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
-               SkbioObject):
+class Sequence(MetadataMixin, PositionalMetadataMixin, IntervalMetadataMixin,
+               collections.Sequence, SkbioObject):
     """Store generic sequence data and optional associated metadata.
 
     ``Sequence`` objects do not enforce an alphabet or grammar and are thus the
@@ -56,6 +53,9 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         constructor. Each column of metadata must be the same length as
         `sequence`. A shallow copy of the positional metadata will be made if
         necessary (see Examples section below for details).
+    interval_metadata : IntervalMetadata
+        Arbitrary metadata which applies to intervals within a sequence to
+        store interval features (such as genes, ncRNA on the sequence).
     lowercase : bool or str, optional
         If ``True``, lowercase sequence characters will be converted to
         uppercase characters. If ``False``, no characters will be converted.
@@ -69,6 +69,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
     values
     metadata
     positional_metadata
+    interval_metadata
     observed_chars
 
     See Also
@@ -88,6 +89,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
     --------
     >>> from pprint import pprint
     >>> from skbio import Sequence
+    >>> from skbio.metadata import IntervalMetadata
 
     **Creating sequences:**
 
@@ -107,8 +109,11 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
     >>> metadata = {'id':'seq-id', 'desc':'seq desc', 'authors': ['Alice']}
     >>> positional_metadata = {'quality': [3, 3, 4, 10],
     ...                        'exons': [True, True, False, True]}
+    >>> interval_metadata = IntervalMetadata(4)
+    >>> interval = interval_metadata.add([(1, 3)], metadata={'gene': 'sagA'})
     >>> seq = Sequence('ACGT', metadata=metadata,
-    ...                positional_metadata=positional_metadata)
+    ...                positional_metadata=positional_metadata,
+    ...                interval_metadata=interval_metadata)
     >>> seq
     Sequence
     -----------------------------
@@ -119,6 +124,8 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
     Positional metadata:
         'exons': <dtype: bool>
         'quality': <dtype: int64>
+    Interval metadata:
+        1 interval feature
     Stats:
         length: 4
     -----------------------------
@@ -160,6 +167,14 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
     2  False        4
     3   True       10
 
+    Retrieve interval metadata:
+
+    >>> seq.interval_metadata   # doctest: +ELLIPSIS
+    1 interval feature
+    ------------------
+    Interval(interval_metadata=<...>, bounds=[(1, 3)], \
+fuzzy=[(False, False)], metadata={'gene': 'sagA'})
+
     **Updating sequence metadata:**
 
     .. warning:: Be aware that a shallow copy of ``metadata`` and
@@ -319,6 +334,32 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
     2      []        4
     3      []       10
 
+    You can also update the interval metadata. Let's re-create a
+    ``Sequence`` object with interval metadata at first:
+
+    >>> seq = Sequence('ACGT')
+    >>> interval = seq.interval_metadata.add(
+    ...     [(1, 3)], metadata={'gene': 'foo'})
+
+    You can update directly on the ``Interval`` object:
+
+    >>> interval  # doctest: +ELLIPSIS
+    Interval(interval_metadata=<...>, bounds=[(1, 3)], \
+fuzzy=[(False, False)], metadata={'gene': 'foo'})
+    >>> interval.bounds = [(0, 2)]
+    >>> interval  # doctest: +ELLIPSIS
+    Interval(interval_metadata=<...>, bounds=[(0, 2)], \
+fuzzy=[(False, False)], metadata={'gene': 'foo'})
+
+    You can also query and obtain the interval features you are
+    interested and then modify them:
+
+    >>> intervals = list(seq.interval_metadata.query(metadata={'gene': 'foo'}))
+    >>> intervals[0].fuzzy = [(True, False)]
+    >>> print(intervals[0])  # doctest: +ELLIPSIS
+    Interval(interval_metadata=<...>, bounds=[(0, 2)], \
+fuzzy=[(True, False)], metadata={'gene': 'foo'})
+
     """
     _number_of_extended_ascii_codes = 256
     # ASCII is built such that the difference between uppercase and lowercase
@@ -348,6 +389,29 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         """
         return self._bytes.view('|S1')
 
+    @property
+    def __array_interface__(self):
+        """Array interface for compatibility with numpy.
+
+        This property allows a ``Sequence`` object to share its underlying data
+        buffer (``Sequence.values``) with numpy. See [1]_ for more details.
+
+        References
+        ----------
+        .. [1] http://docs.scipy.org/doc/numpy/reference/arrays.interface.html
+
+        Examples
+        --------
+        >>> import numpy as np
+        >>> from skbio import Sequence
+        >>> seq = Sequence('ABC123')
+        >>> np.asarray(seq) # doctest: +NORMALIZE_WHITESPACE
+        array([b'A', b'B', b'C', b'1', b'2', b'3'],
+              dtype='|S1')
+
+        """
+        return self.values.__array_interface__
+
     @property
     @experimental(as_of="0.4.1")
     def observed_chars(self):
@@ -378,7 +442,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
 
         Parameters
         ----------
-        seqs : iterable (Sequence)
+        sequences : iterable (Sequence)
             An iterable of ``Sequence`` objects or appropriate subclasses.
         how : {'strict', 'inner', 'outer'}, optional
             How to intersect the `positional_metadata` of the sequences.
@@ -423,14 +487,14 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         >>> s2 = DNA("GGAA")
         >>> DNA.concat([s1, s2])
         DNA
-        -----------------------------
+        --------------------------
         Stats:
             length: 8
             has gaps: False
             has degenerates: False
-            has non-degenerates: True
+            has definites: True
             GC-content: 50.00%
-        -----------------------------
+        --------------------------
         0 ACGTGGAA
 
         Concatenate DNA sequences into a Sequence object (type coercion):
@@ -451,7 +515,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         >>> result = DNA.concat([s1, s2], how='outer')
         >>> result
         DNA
-        -----------------------------
+        ---------------------------
         Positional metadata:
             'one': <dtype: bool>
             'two': <dtype: float64>
@@ -459,9 +523,9 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
             length: 8
             has gaps: False
             has degenerates: False
-            has non-degenerates: True
+            has definites: True
             GC-content: 50.00%
-        -----------------------------
+        ---------------------------
         0 ACGTGGAA
         >>> result.positional_metadata
              one  two
@@ -469,10 +533,10 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         1   True  NaN
         2   True  NaN
         3  False  NaN
-        4  False    1
-        5  False    2
-        6   True    3
-        7  False    4
+        4  False  1.0
+        5  False  2.0
+        6   True  3.0
+        7  False  4.0
 
         """
         if how not in {'strict', 'inner', 'outer'}:
@@ -487,13 +551,13 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
 
         if how == 'strict':
             how = 'inner'
-            cols = []
+            cols = set()
             for s in seqs:
                 if s.has_positional_metadata():
-                    cols.append(frozenset(s.positional_metadata))
+                    cols.add(frozenset(s.positional_metadata))
                 else:
-                    cols.append(frozenset())
-            if len(set(cols)) > 1:
+                    cols.add(frozenset())
+            if len(cols) > 1:
                 raise ValueError("The positional metadata of the sequences do"
                                  " not have matching columns. Consider setting"
                                  " how='inner' or how='outer'")
@@ -508,7 +572,9 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         pm = pd.concat(pm_data, join=how, ignore_index=True)
         bytes_ = np.concatenate(seq_data)
 
-        return cls(bytes_, positional_metadata=pm)
+        im = IntervalMetadata.concat(i.interval_metadata for i in seqs)
+
+        return cls(bytes_, positional_metadata=pm, interval_metadata=im)
 
     @classmethod
     def _assert_can_cast_to(cls, target):
@@ -520,9 +586,13 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
     def _positional_metadata_axis_len_(self):
         return len(self)
 
+    @overrides(IntervalMetadataMixin)
+    def _interval_metadata_axis_len_(self):
+        return len(self)
+
     @stable(as_of="0.4.0")
     def __init__(self, sequence, metadata=None, positional_metadata=None,
-                 lowercase=False):
+                 interval_metadata=None, lowercase=False):
         if isinstance(sequence, np.ndarray):
             if sequence.dtype == np.uint8:
                 self._set_bytes_contiguous(sequence)
@@ -542,25 +612,21 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
             # Sequence casting is acceptable between direct
             # decendants/ancestors
             sequence._assert_can_cast_to(type(self))
-            # we're not simply accessing sequence.metadata in order to avoid
-            # creating "empty" metadata representations on both sequence
-            # objects if they don't have metadata. same strategy is used below
-            # for positional metadata
+
             if metadata is None and sequence.has_metadata():
                 metadata = sequence.metadata
             if (positional_metadata is None and
                     sequence.has_positional_metadata()):
                 positional_metadata = sequence.positional_metadata
+            if (interval_metadata is None and
+                    sequence.has_interval_metadata()):
+                interval_metadata = sequence.interval_metadata
             sequence = sequence._bytes
-
             self._owns_bytes = False
-
             self._set_bytes(sequence)
-
         else:
-            # Python 3 will not raise a UnicodeEncodeError so we force it by
-            # encoding it as ascii
-            if isinstance(sequence, six.text_type):
+            # Encode as ascii to raise UnicodeEncodeError if necessary.
+            if isinstance(sequence, str):
                 sequence = sequence.encode("ascii")
             s = np.fromstring(sequence, dtype=np.uint8)
 
@@ -580,10 +646,12 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         MetadataMixin._init_(self, metadata=metadata)
         PositionalMetadataMixin._init_(
             self, positional_metadata=positional_metadata)
+        IntervalMetadataMixin._init_(
+            self, interval_metadata=interval_metadata)
 
         if lowercase is False:
             pass
-        elif lowercase is True or isinstance(lowercase, six.string_types):
+        elif lowercase is True or isinstance(lowercase, str):
             lowercase_mask = self._bytes > self._ascii_lowercase_boundary
             self._convert_to_uppercase(lowercase_mask)
 
@@ -693,13 +761,14 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         False
 
         Define a sequence with the same sequence of characters as ``u`` but
-        with different metadata and positional metadata:
+        with different metadata, positional metadata, and interval metadata:
 
         >>> v = Sequence('ACGA', metadata={'id': 'abc'},
         ...              positional_metadata={'quality':[1, 5, 3, 3]})
+        >>> _ = v.interval_metadata.add([(0, 1)])
 
-        The two sequences are not considered equal because their metadata and
-        positional metadata do not match:
+        The two sequences are not considered equal because their metadata,
+        positional metadata, and interval metadata do not match:
 
         >>> u == v
         False
@@ -718,6 +787,9 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         if not PositionalMetadataMixin._eq_(self, other):
             return False
 
+        if not IntervalMetadataMixin._eq_(self, other):
+            return False
+
         return True
 
     @stable(as_of="0.4.0")
@@ -758,6 +830,11 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
     def __getitem__(self, indexable):
         """Slice this sequence.
 
+        Notes
+        -----
+        This drops the ``self.interval_metadata`` from the returned
+        new ``Sequence`` object.
+
         Parameters
         ----------
         indexable : int, slice, iterable (int and slice), 1D array_like (bool)
@@ -824,7 +901,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
 
         """
         if (not isinstance(indexable, np.ndarray) and
-            ((not isinstance(indexable, six.string_types)) and
+            ((not isinstance(indexable, str)) and
              hasattr(indexable, '__iter__'))):
             indexable_ = indexable
             indexable = np.asarray(indexable)
@@ -847,9 +924,15 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
                                              self.positional_metadata, index))
                         positional_metadata = pd.concat(pos_md_slices)
 
-                    return self._to(sequence=seq,
-                                    positional_metadata=positional_metadata)
-        elif (isinstance(indexable, six.string_types) or
+                    metadata = None
+                    if self.has_metadata():
+                        metadata = self.metadata
+
+                    return self._constructor(
+                        sequence=seq,
+                        metadata=metadata,
+                        positional_metadata=positional_metadata)
+        elif (isinstance(indexable, str) or
                 isinstance(indexable, bool)):
             raise IndexError("Cannot index with %s type: %r" %
                              (type(indexable).__name__, indexable))
@@ -869,7 +952,14 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         seq = self._bytes[indexable]
         positional_metadata = self._slice_positional_metadata(indexable)
 
-        return self._to(sequence=seq, positional_metadata=positional_metadata)
+        metadata = None
+        if self.has_metadata():
+            metadata = self.metadata
+
+        return self._constructor(
+            sequence=seq,
+            metadata=metadata,
+            positional_metadata=positional_metadata)
 
     def _slice_positional_metadata(self, indexable):
         if self.has_positional_metadata():
@@ -920,8 +1010,6 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         """
         return len(self) > 0
 
-    __nonzero__ = __bool__
-
     @stable(as_of="0.4.0")
     def __iter__(self):
         """Iterate over positions in this sequence.
@@ -1010,6 +1098,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
           in the order they appear in the positional metadata ``pd.DataFrame``.
           Column names (i.e., keys) follow the same display rules as metadata
           keys
+        * interval metadata: the number of interval features will be displayed.
         * sequence stats (e.g., length)
         * up to five lines of chunked sequence data. Each line of chunked
           sequence data displays the current position in the sequence
@@ -1029,6 +1118,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         Short sequence without metadata:
 
         >>> from skbio import Sequence
+        >>> from skbio.metadata._interval import IntervalMetadata
         >>> Sequence('ACGTAATGGATACGTAATGCA')
         Sequence
         -------------------------
@@ -1051,7 +1141,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         300 ACGTACGTAC GTACGTACGT ACGTACGTAC GTACGTACGT ACGTACGTAC GTACGTACGT
         360 ACGTACGTAC GTACGTACGT ACGTACGTAC GTACGTACGT
 
-        Sequence with metadata and positional metadata:
+        Sequence with metadata, positional metadata, and interval metadata:
 
         >>> metadata = {
         ...     'id': 'seq-id',
@@ -1065,8 +1155,10 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         ...     'quality': [3, 10, 11, 10],
         ...     'exons': [True, True, False, True]
         ... }
-        >>> Sequence('ACGT', metadata=metadata,
+        >>> seq = Sequence('ACGT', metadata=metadata,
         ...          positional_metadata=positional_metadata)
+        >>> _ = seq.interval_metadata.add([(0, 2)], metadata={'gene': 'sagA'})
+        >>> seq
         Sequence
         ----------------------------------------------------------------------
         Metadata:
@@ -1079,6 +1171,8 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         Positional metadata:
             'exons': <dtype: bool>
             'quality': <dtype: int64>
+        Interval metadata:
+            1 interval feature
         Stats:
             length: 4
         ----------------------------------------------------------------------
@@ -1141,129 +1235,6 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         """
         return self._copy(True, memo)
 
-    @deprecated(as_of="0.4.1", until="0.5.1",
-                reason="Use `copy.copy(seq)` instead of "
-                       "`seq.copy(deep=False)`, and `copy.deepcopy(seq)` "
-                       "instead of `seq.copy(deep=True)`.")
-    def copy(self, deep=False):
-        """Return a copy of this sequence.
-
-        Parameters
-        ----------
-        deep : bool, optional
-            Perform a deep copy. If ``False``, perform a shallow copy.
-
-        Returns
-        -------
-        Sequence
-            Copy of this sequence.
-
-        Notes
-        -----
-        Since sequence objects can share the same underlying immutable sequence
-        data (or pieces of it), this method can be used to create a sequence
-        object with its own copy of the sequence data so that the original
-        sequence data can be garbage-collected.
-
-        Examples
-        --------
-        Create a sequence:
-
-        >>> from pprint import pprint
-        >>> from skbio import Sequence
-        >>> seq = Sequence('ACGT',
-        ...                metadata={'id': 'seq-id', 'authors': ['Alice']},
-        ...                positional_metadata={'quality': [7, 10, 8, 5],
-        ...                                     'list': [[], [], [], []]})
-
-        Make a shallow copy of the sequence:
-
-        >>> seq_copy = seq.copy()
-        >>> seq_copy == seq
-        True
-
-        Setting new references in the copied sequence's metadata doesn't affect
-        the original sequence's metadata:
-
-        >>> seq_copy.metadata['id'] = 'new-id'
-        >>> pprint(seq_copy.metadata)
-        {'authors': ['Alice'], 'id': 'new-id'}
-        >>> pprint(seq.metadata)
-        {'authors': ['Alice'], 'id': 'seq-id'}
-
-        The same applies to the sequence's positional metadata:
-
-        >>> seq_copy.positional_metadata.loc[0, 'quality'] = 999
-        >>> seq_copy.positional_metadata
-          list  quality
-        0   []      999
-        1   []       10
-        2   []        8
-        3   []        5
-        >>> seq.positional_metadata
-          list  quality
-        0   []        7
-        1   []       10
-        2   []        8
-        3   []        5
-
-        Since only a *shallow* copy was made, updates to mutable objects stored
-        as metadata affect the original sequence's metadata:
-
-        >>> seq_copy.metadata['authors'].append('Bob')
-        >>> pprint(seq_copy.metadata)
-        {'authors': ['Alice', 'Bob'], 'id': 'new-id'}
-        >>> pprint(seq.metadata)
-        {'authors': ['Alice', 'Bob'], 'id': 'seq-id'}
-
-        The same applies to the sequence's positional metadata:
-
-        >>> seq_copy.positional_metadata.loc[0, 'list'].append(1)
-        >>> seq_copy.positional_metadata
-          list  quality
-        0  [1]      999
-        1   []       10
-        2   []        8
-        3   []        5
-        >>> seq.positional_metadata
-          list  quality
-        0  [1]        7
-        1   []       10
-        2   []        8
-        3   []        5
-
-        Perform a deep copy to avoid this behavior:
-
-        >>> seq_deep_copy = seq.copy(deep=True)
-
-        Updates to mutable objects no longer affect the original sequence's
-        metadata:
-
-        >>> seq_deep_copy.metadata['authors'].append('Carol')
-        >>> pprint(seq_deep_copy.metadata)
-        {'authors': ['Alice', 'Bob', 'Carol'], 'id': 'seq-id'}
-        >>> pprint(seq.metadata)
-        {'authors': ['Alice', 'Bob'], 'id': 'seq-id'}
-
-        Nor its positional metadata:
-
-        >>> seq_deep_copy.positional_metadata.loc[0, 'list'].append(2)
-        >>> seq_deep_copy.positional_metadata
-             list  quality
-        0  [1, 2]        7
-        1      []       10
-        2      []        8
-        3      []        5
-        >>> seq.positional_metadata
-          list  quality
-        0  [1]        7
-        1   []       10
-        2   []        8
-        3   []        5
-
-        """
-        return self._copy(deep, {})
-
     def _copy(self, deep, memo):
         # strategy: copy the sequence without metadata first, then set metadata
         # attributes with copies. we take this approach instead of simply
@@ -1279,16 +1250,21 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         bytes_ = np.copy(self._bytes)
 
         seq_copy = self._constructor(sequence=bytes_, metadata=None,
-                                     positional_metadata=None)
+                                     positional_metadata=None,
+                                     interval_metadata=None)
 
         if deep:
             seq_copy._metadata = MetadataMixin._deepcopy_(self, memo)
             seq_copy._positional_metadata = \
                 PositionalMetadataMixin._deepcopy_(self, memo)
+            seq_copy._interval_metadata = IntervalMetadataMixin._deepcopy_(
+                self, memo)
         else:
             seq_copy._metadata = MetadataMixin._copy_(self)
             seq_copy._positional_metadata = \
                 PositionalMetadataMixin._copy_(self)
+            seq_copy._interval_metadata = IntervalMetadataMixin._copy_(
+                self)
 
         return seq_copy
 
@@ -1386,6 +1362,91 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         return self._string.count(
             self._munge_to_bytestring(subsequence, "count"), start, end)
 
+    @experimental(as_of="0.5.0")
+    def replace(self, where, character):
+        """Replace values in this sequence with a different character.
+
+        Parameters
+        ----------
+        where : 1D array_like (bool) or iterable (slices or ints) or str
+            Indicates positions in the sequence to replace with `character`.
+            Can be a boolean vector, an iterable of indices/slices, or a
+            string that is a key in `positional_metadata` pointing to a
+            boolean vector.
+        character : str or bytes
+            Character that will replace chosen items in this sequence.
+
+        Returns
+        -------
+        Sequence
+            Copy of this sequence, with chosen items replaced with chosen
+            character. All metadata is retained.
+
+        Examples
+        --------
+        Let's create and display a Sequence:
+
+        >>> from skbio import Sequence
+        >>> sequence = Sequence('GGTACCAACG')
+        >>> str(sequence)
+        'GGTACCAACG'
+
+        Let's call ``replace`` on the Sequence using a boolean vector for
+        ``where`` and assign it to a new variable:
+
+        >>> seq = sequence.replace([False, False, False, True, False, False,
+        ...                         True, True, False, False], '-')
+
+        Let's take a look at the new Sequence:
+
+        >>> str(seq)
+        'GGT-CC--CG'
+
+        Other types of input are accepted by the ``where`` parameter. Let's
+        pass in a list of indices and slices that is equivalent to the boolean
+        vector we used previously:
+
+        >>> str(seq) == str(sequence.replace([3, slice(6, 8)], '-'))
+        True
+
+        ``where`` also accepts a boolean vector contained in
+        ``Sequence.positional_metadata``:
+
+        >>> sequence.positional_metadata = {'where':
+        ...                                 [False, False, False, True, False,
+        ...                                  False, True, True, False, False]}
+
+        Let's pass in the key ``'where'`` and compare to ``seq``:
+
+        >>> str(seq) == str(sequence.replace('where', '-'))
+        True
+
+        """
+        if type(character) is not bytes:
+            character = character.encode('ascii')
+        character = ord(character)
+        index = self._munge_to_index_array(where)
+        seq_bytes = self._bytes.copy()
+        seq_bytes[index] = character
+
+        metadata = None
+        if self.has_metadata():
+            metadata = self.metadata
+
+        positional_metadata = None
+        if self.has_positional_metadata():
+            positional_metadata = self.positional_metadata
+
+        interval_metadata = None
+        if self.has_interval_metadata():
+            interval_metadata = self.interval_metadata
+        # Use __class__ instead of _constructor so that validations are
+        # performed for subclasses (the user could have introduced invalid
+        # characters).
+        return self.__class__(seq_bytes, metadata=metadata,
+                              positional_metadata=positional_metadata,
+                              interval_metadata=interval_metadata)
+
     @stable(as_of="0.4.0")
     def index(self, subsequence, start=None, end=None):
         """Find position where subsequence first occurs in the sequence.
@@ -1434,70 +1495,61 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         Parameters
         ----------
         other : str, Sequence, or 1D np.ndarray (np.uint8 or '\|S1')
-            Sequence to compute the distance to.
+            Sequence to compute the distance to. If `other` is a ``Sequence``
+            object, it must be the same type as this sequence. Other input
+            types will be converted into a ``Sequence`` object of the same type
+            as this sequence.
         metric : function, optional
             Function used to compute the distance between this sequence and
-            `other`. If ``None`` (the default),
-            ``scipy.spatial.distance.hamming`` will be used. This function
-            should take two ``skbio.Sequence`` objects and return a ``float``.
+            `other`. If ``None`` (the default), Hamming distance will be used
+            (:func:`skbio.sequence.distance.hamming`). `metric` should take two
+            ``skbio.Sequence`` objects and return a ``float``. The sequence
+            objects passed to `metric` will be the same type as this sequence.
+            See :mod:`skbio.sequence.distance` for other predefined metrics
+            that can be supplied via `metric`.
 
         Returns
         -------
         float
-            Distance between this sequence and `other`.
+            Distance between this sequence and `other` as defined by `metric`.
 
         Raises
         ------
-        ValueError
-            If the sequences are not the same length when `metric` is ``None``
-            (i.e., `metric` is ``scipy.spatial.distance.hamming``). This is
-            only checked when using this metric, as equal length is not a
-            requirement of all sequence distance metrics. In general, the
-            metric itself should test and give an informative error message,
-            but the message from ``scipy.spatial.distance.hamming`` is somewhat
-            cryptic (as of this writing), and it's the default metric, so we
-            explicitly do this check here. This metric-specific check will be
-            removed from this method when the ``skbio.sequence.stats`` module
-            is created (track progress on issue #913).
         TypeError
             If `other` is a ``Sequence`` object with a different type than this
             sequence.
 
         See Also
         --------
+        skbio.sequence.distance
         fraction_diff
         fraction_same
-        scipy.spatial.distance.hamming
 
         Examples
         --------
         >>> from skbio import Sequence
         >>> s = Sequence('GGUC')
         >>> t = Sequence('AGUC')
+
+        Compute Hamming distance (the default metric):
+
         >>> s.distance(t)
         0.25
-        >>> def custom_dist(s1, s2): return 0.42
-        >>> s.distance(t, custom_dist)
+
+        Use a custom metric:
+
+        >>> def custom_metric(s1, s2): return 0.42
+        >>> s.distance(t, custom_metric)
         0.42
 
         """
         # TODO refactor this method to accept a name (string) of the distance
         # metric to apply and accept **kwargs
-        other = self._munge_to_sequence(other, 'distance')
+        other = self._munge_to_self_type(other, 'distance')
         if metric is None:
-            return self._hamming(other)
+            metric = skbio.sequence.distance.hamming
         return float(metric(self, other))
 
-    def _hamming(self, other):
-        # Hamming requires equal length sequences. We are checking this
-        # here because the error you would get otherwise is cryptic.
-        if len(self) != len(other):
-            raise ValueError(
-                "Sequences do not have equal length. "
-                "Hamming distances can only be computed between "
-                "sequences of equal length.")
-        return float(hamming(self.values, other.values))
-
     @stable(as_of="0.4.0")
     def matches(self, other):
         """Find positions that match with another sequence.
@@ -1780,8 +1832,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
 
     def _chars_to_indices(self, chars):
         """Helper for Sequence.frequencies."""
-        if isinstance(chars, six.string_types) or \
-                isinstance(chars, six.binary_type):
+        if isinstance(chars, (str, bytes)):
             chars = set([chars])
         elif not isinstance(chars, set):
             raise TypeError(
@@ -1792,8 +1843,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         chars = list(chars)
         indices = []
         for char in chars:
-            if not (isinstance(char, six.string_types) or
-                    isinstance(char, six.binary_type)):
+            if not isinstance(char, (str, bytes)):
                 raise TypeError(
                     "Each element of `chars` must be string-like, not %r" %
                     type(char).__name__)
@@ -1861,15 +1911,25 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
             step = k
             count = len(self) // k
 
-        if self.has_positional_metadata():
+        if len(self) == 0 or self.has_positional_metadata():
+            # Slower path when sequence is empty or positional metadata needs
+            # to be sliced.
             for i in range(0, len(self) - k + 1, step):
                 yield self[i:i+k]
-        # Optimized path when no positional metadata
         else:
+            # Optimized path when positional metadata doesn't need slicing.
             kmers = np.lib.stride_tricks.as_strided(
                 self._bytes, shape=(k, count), strides=(1, step)).T
+
+            metadata = None
+            if self.has_metadata():
+                metadata = self.metadata
+
             for s in kmers:
-                yield self._to(sequence=s)
+                yield self._constructor(
+                    sequence=s,
+                    metadata=metadata,
+                    positional_metadata=None)
 
     @stable(as_of="0.4.0")
     def kmer_frequencies(self, k, overlap=True, relative=False):
@@ -1918,7 +1978,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
                 num_kmers = len(self) // k
 
             relative_freqs = {}
-            for kmer, count in viewitems(freqs):
+            for kmer, count in freqs.items():
                 relative_freqs[kmer] = count / num_kmers
             freqs = relative_freqs
 
@@ -1955,7 +2015,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         'TATAA'
 
         """
-        if isinstance(regex, six.string_types):
+        if isinstance(regex, str):
             regex = re.compile(regex)
 
         lookup = np.arange(len(self))
@@ -2042,53 +2102,6 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
             if len(r) >= min_length:
                 yield r
 
-    def _to(self, sequence=None, metadata=None, positional_metadata=None):
-        """Return a copy of this sequence.
-
-        Returns a copy of this sequence, optionally with updated attributes
-        specified as keyword arguments.
-
-        Arguments are the same as those passed to the ``Sequence`` constructor.
-        The returned copy will have its attributes updated based on the
-        arguments. If an attribute is missing, the copy will keep the same
-        attribute as this sequence. Valid attribute names are `'sequence'`,
-        `'metadata'`, and `'positional_metadata'`. Default behavior is to
-        return a copy of this sequence without changing any attributes.
-
-        Parameters
-        ----------
-        sequence : optional
-        metadata : optional
-        positional_metadata : optional
-
-        Returns
-        -------
-        Sequence
-            Copy of this sequence, optionally with updated attributes based on
-            arguments. Will be the same type as this sequence (`self`).
-
-        Notes
-        -----
-        By default, `metadata` and `positional_metadata` are shallow-copied and
-        the reference to `sequence` is used (without copying) for efficiency
-        since `sequence` is immutable. This differs from the behavior of
-        `Sequence.copy`, which will actually copy `sequence`.
-
-        This method is the preferred way of creating new instances from an
-        existing sequence, instead of calling ``self.__class__(...)``, as the
-        latter can be error-prone (e.g., it's easy to forget to propagate
-        attributes to the new instance).
-
-        """
-        if sequence is None:
-            sequence = self._bytes
-        if metadata is None and self.has_metadata():
-            metadata = self._metadata
-        if positional_metadata is None and self.has_positional_metadata():
-            positional_metadata = self._positional_metadata
-        return self._constructor(sequence=sequence, metadata=metadata,
-                                 positional_metadata=positional_metadata)
-
     def _constructor(self, **kwargs):
         return self.__class__(**kwargs)
 
@@ -2096,7 +2109,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
         """Return an index array from something isomorphic to a boolean vector.
 
         """
-        if isinstance(sliceable, six.string_types):
+        if isinstance(sliceable, str):
             if sliceable in self.positional_metadata:
                 if self.positional_metadata[sliceable].dtype == np.bool:
                     sliceable = self.positional_metadata[sliceable]
@@ -2145,6 +2158,17 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
 
         return normalized
 
+    def _munge_to_self_type(self, other, method):
+        if isinstance(other, Sequence):
+            if type(other) != type(self):
+                raise TypeError("Cannot use %s and %s together with `%s`" %
+                                (self.__class__.__name__,
+                                 other.__class__.__name__, method))
+            else:
+                return other
+
+        return self.__class__(other)
+
     def _munge_to_sequence(self, other, method):
         if isinstance(other, Sequence):
             if type(other) != type(self):
@@ -2162,7 +2186,7 @@ class Sequence(MetadataMixin, PositionalMetadataMixin, collections.Sequence,
     def _munge_to_bytestring(self, other, method):
         if type(other) is bytes:
             return other
-        elif isinstance(other, six.string_types):
+        elif isinstance(other, str):
             return other.encode('ascii')
         else:
             return self._munge_to_sequence(other, method)._string
