@@ -6,15 +6,15 @@
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
-
 import unittest
 
-import six
 import numpy as np
 
 from skbio import DNA, RNA, Protein, GeneticCode
 from skbio.sequence._nucleotide_mixin import NucleotideMixin
+from skbio.sequence import GrammaredSequence
+from skbio.util import classproperty
+from skbio.metadata import IntervalMetadata
 
 
 # This file contains tests for functionality of sequence types which implement
@@ -22,7 +22,7 @@ from skbio.sequence._nucleotide_mixin import NucleotideMixin
 # similar that the testing logic can be shared and parameterized across
 # different test data.
 
-class TestNucelotideSequence(unittest.TestCase):
+class TestNucleotideSequence(unittest.TestCase):
     def setUp(self):
         self.sequence_kinds = frozenset([
             str,
@@ -66,6 +66,7 @@ class TestNucelotideSequence(unittest.TestCase):
         self.assertIn("abstract class", str(cm.exception))
         self.assertIn("complement_map", str(cm.exception))
 
+    # TODO: remove when nondegenerate_chars is removed
     def test_nondegenerate_chars(self):
         dna = (DNA, "ACGT")
         rna = (RNA, "ACGU")
@@ -74,6 +75,14 @@ class TestNucelotideSequence(unittest.TestCase):
             self.assertEqual(constructor('').nondegenerate_chars, exp)
             self.assertEqual(constructor.nondegenerate_chars, exp)
 
+    def test_definite_chars(self):
+        dna = (DNA, "ACGT")
+        rna = (RNA, "ACGU")
+        for constructor, definite_char in (dna, rna):
+            exp = set(definite_char)
+            self.assertEqual(constructor('').definite_chars, exp)
+            self.assertEqual(constructor.definite_chars, exp)
+
     def test_degenerate_map(self):
         dna_exp = (DNA, {
             'B': set(['C', 'T', 'G']), 'D': set(['A', 'T', 'G']),
@@ -165,7 +174,7 @@ class TestNucelotideSequence(unittest.TestCase):
 
     def test_translate_invalid_id(self):
         for seq in RNA('AUG'), DNA('ATG'):
-            with six.assertRaisesRegex(self, ValueError, 'table_id.*42'):
+            with self.assertRaisesRegex(ValueError, 'table_id.*42'):
                 seq.translate(42)
 
     def test_translate_six_frames_ncbi_table_id(self):
@@ -220,7 +229,7 @@ class TestNucelotideSequence(unittest.TestCase):
 
     def test_translate_six_frames_invalid_id(self):
         for seq in RNA('AUG'), DNA('ATG'):
-            with six.assertRaisesRegex(self, ValueError, 'table_id.*42'):
+            with self.assertRaisesRegex(ValueError, 'table_id.*42'):
                 seq.translate_six_frames(42)
 
     def test_repr(self):
@@ -264,7 +273,8 @@ class TestNucelotideSequence(unittest.TestCase):
             comp = constructor(
                 '',
                 metadata={'id': 'foo', 'description': 'bar'},
-                positional_metadata={'quality': []}).complement()
+                positional_metadata={'quality': []},
+                interval_metadata=IntervalMetadata(0)).complement()
             self.assertEqual(
                 comp,
                 constructor(
@@ -278,16 +288,20 @@ class TestNucelotideSequence(unittest.TestCase):
             comp = constructor(seq_str).complement()
             self.assertEqual(comp, constructor(comp_str))
 
+            im = IntervalMetadata(len(seq_str))
+            im.add([(0, 1)], metadata={'gene': 'p53'})
             comp = constructor(
                 seq_str,
                 metadata={'id': 'foo', 'description': 'bar'},
-                positional_metadata={'quality': qual}).complement()
+                positional_metadata={'quality': qual},
+                interval_metadata=im).complement()
             self.assertEqual(
                 comp,
                 constructor(
                     comp_str,
                     metadata={'id': 'foo', 'description': 'bar'},
-                    positional_metadata={'quality': qual}))
+                    positional_metadata={'quality': qual},
+                    interval_metadata=im))
 
     def test_complement_with_reverse_empty(self):
         for constructor in (DNA, RNA):
@@ -297,7 +311,8 @@ class TestNucelotideSequence(unittest.TestCase):
             rc = constructor(
                 '',
                 metadata={'id': 'foo', 'description': 'bar'},
-                positional_metadata={'quality': []}).complement(reverse=True)
+                positional_metadata={'quality': []},
+                interval_metadata=IntervalMetadata(0)).complement(reverse=True)
             self.assertEqual(
                 rc,
                 constructor(
@@ -311,18 +326,30 @@ class TestNucelotideSequence(unittest.TestCase):
             rc = constructor(seq_str).complement(reverse=True)
             self.assertEqual(rc, constructor(rev_comp_str))
 
-            rc = constructor(
+            l = len(seq_str)
+            im = IntervalMetadata(l)
+            im.add([(0, 1)], metadata={'gene': 'p53'})
+            im_rc = IntervalMetadata(l)
+            im_rc.add([(l-1, l)], metadata={'gene': 'p53'})
+            original = constructor(
                 seq_str,
                 metadata={'id': 'foo', 'description': 'bar'},
                 positional_metadata={
-                    'quality': qual}).complement(reverse=True)
+                    'quality': qual},
+                interval_metadata=im)
+            rc = original.complement(reverse=True)
+
             self.assertEqual(
                 rc,
                 constructor(
                     rev_comp_str,
                     metadata={'id': 'foo', 'description': 'bar'},
                     positional_metadata={'quality':
-                                         list(qual)[::-1]}))
+                                         list(qual)[::-1]},
+                    interval_metadata=im_rc))
+            # assert the original object is not changed
+            self.assertIsNot(original.interval_metadata, im)
+            self.assertEqual(original.interval_metadata, im)
 
     def test_reverse_complement(self):
         # light tests because this just calls
@@ -408,12 +435,29 @@ class TestNucelotideSequence(unittest.TestCase):
 
     def test_is_reverse_complement_type_mismatch(self):
         for Class in (DNA, RNA):
-            class Subclass(Class):
-                pass
-            seq1 = Class('ABC')
-            seq2 = Subclass('ABC')
+            class DifferentSequenceClass(GrammaredSequence):
+                @classproperty
+                def degenerate_map(cls):
+                    return {"X": set("AB")}
 
-            with self.assertRaises(TypeError):
+                @classproperty
+                def definite_chars(cls):
+                    return set("ABC")
+
+                @classproperty
+                def default_gap_char(cls):
+                    return '-'
+
+                @classproperty
+                def gap_chars(cls):
+                    return set('-.')
+
+            seq1 = Class('ABC')
+            seq2 = DifferentSequenceClass('ABC')
+
+            with self.assertRaisesRegex(TypeError,
+                                        "Cannot use.*and "
+                                        "DifferentSequenceClass together"):
                 seq1.is_reverse_complement(seq2)
 
     def test_motif_purine_run(self):
