@@ -6,8 +6,6 @@
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
-
 import unittest
 
 import numpy as np
@@ -29,11 +27,18 @@ class TestProtein(unittest.TestCase):
         with self.assertRaises(AttributeError):
             Protein('').alphabet = set("ABCD")
 
+    # TODO: duplicate of test_definite_chars, remove when nondegenerate_chars,
+    # is removed
     def test_nondegenerate_chars(self):
         exp = set("ACDEFGHIKLMNPQRSTVWY")
         self.assertEqual(Protein("").nondegenerate_chars, exp)
         self.assertEqual(Protein.nondegenerate_chars, exp)
 
+    def test_definite_chars(self):
+        exp = set("ACDEFGHIKLMNPQRSTVWY")
+        self.assertEqual(Protein("").definite_chars, exp)
+        self.assertEqual(Protein.definite_chars, exp)
+
     def test_degenerate_map(self):
         exp = {
             'B': set(['D', 'N']), 'Z': set(['E', 'Q']),
@@ -119,6 +124,11 @@ class TestProtein(unittest.TestCase):
         self.assertIn('has gaps: False', obs)
         self.assertIn('has stops: True', obs)
 
+    def test_cannot_subclass(self):
+        with self.assertRaisesRegex(TypeError, "Subclassing disabled"):
+            class CustomSequence(Protein):
+                pass
+
 
 if __name__ == "__main__":
     unittest.main()
