@@ -99,10 +99,10 @@ array([ 0.25,  0.25,  0.5 ])
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
 import numpy as np
 import pandas as pd
 import scipy.stats
+import skbio.util
 from skbio.util._decorator import experimental
 
 
@@ -124,6 +124,15 @@ def closure(mat):
        A matrix of proportions where all of the values
        are nonzero and each composition (row) adds up to 1
 
+    Raises
+    ------
+    ValueError
+       Raises an error if any values are negative.
+    ValueError
+       Raises an error if the matrix has more than 2 dimension.
+    ValueError
+       Raises an error if there is a row that has all zeros.
+
     Examples
     --------
     >>> import numpy as np
@@ -139,6 +148,8 @@ def closure(mat):
         raise ValueError("Cannot have negative proportions")
     if mat.ndim > 2:
         raise ValueError("Input matrix can only have two dimensions or less")
+    if np.all(mat == 0, axis=1).sum() > 0:
+        raise ValueError("Input matrix cannot have rows with all zeros")
     mat = mat / mat.sum(axis=1, keepdims=True)
     return mat.squeeze()
 
@@ -170,6 +181,16 @@ def multiplicative_replacement(mat, delta=None):
        A matrix of proportions where all of the values
        are nonzero and each composition (row) adds up to 1
 
+    Raises
+    ------
+    ValueError
+       Raises an error if negative proportions are created due to a large
+       `delta`.
+
+    Notes
+    -----
+    This method will result in negative proportions if a large delta is chosen.
+
     References
     ----------
     .. [1] J. A. Martin-Fernandez. "Dealing With Zeros and Missing Values in
@@ -196,6 +217,9 @@ def multiplicative_replacement(mat, delta=None):
         delta = (1. / num_feats)**2
 
     zcnts = 1 - tot * delta
+    if np.any(zcnts) < 0:
+        raise ValueError('The multiplicative replacment created negative '
+                         'proportions. Consider using a smaller `delta`.')
     mat = np.where(z_mat, delta, zcnts * mat)
     return mat.squeeze()
 
@@ -381,8 +405,8 @@ def inner(x, y):
     >>> from skbio.stats.composition import inner
     >>> x = np.array([.1, .3, .4, .2])
     >>> y = np.array([.2, .4, .2, .2])
-    >>> inner(x, y)
-    0.21078524737545556
+    >>> inner(x, y)  # doctest: +ELLIPSIS
+    0.2107852473...
     """
     x = closure(x)
     y = closure(y)
@@ -514,7 +538,10 @@ def ilr(mat, basis=None, check=True):
 
     basis: numpy.ndarray, float, optional
         orthonormal basis for Aitchison simplex
-        defaults to J.J.Egozcue orthonormal basis
+        defaults to J.J.Egozcue orthonormal basis.
+
+    check: bool
+        Specifies if the basis is orthonormal.
 
     Examples
     --------
@@ -524,12 +551,24 @@ def ilr(mat, basis=None, check=True):
     >>> ilr(x)
     array([-0.7768362 , -0.68339802,  0.11704769])
 
+    Notes
+    -----
+    If the `basis` parameter is specified, it is expected to be a basis in the
+    Aitchison simplex.  If there are `D-1` elements specified in `mat`, then
+    the dimensions of the basis needs be `D-1 x D`, where rows represent
+    basis vectors, and the columns represent proportions.
     """
     mat = closure(mat)
     if basis is None:
         basis = clr_inv(_gram_schmidt_basis(mat.shape[-1]))
-    elif check:
-        _check_orthogonality(basis)
+    else:
+        if len(basis.shape) != 2:
+            raise ValueError("Basis needs to be a 2D matrix, "
+                             "not a %dD matrix." %
+                             (len(basis.shape)))
+        if check:
+            _check_orthogonality(basis)
+
     return inner(mat, basis)
 
 
@@ -551,7 +590,7 @@ def ilr_inv(mat, basis=None, check=True):
 
     where :math:`[e_1,\ldots, e_{D-1}]` is an orthonormal basis in the simplex.
 
-    If an orthornormal basis isn't specified, the J. J. Egozcue orthonormal
+    If an orthonormal basis isn't specified, the J. J. Egozcue orthonormal
     basis derived from Gram-Schmidt orthogonalization will be used by
     default.
 
@@ -567,6 +606,10 @@ def ilr_inv(mat, basis=None, check=True):
         orthonormal basis for Aitchison simplex
         defaults to J.J.Egozcue orthonormal basis
 
+    check: bool
+        Specifies if the basis is orthonormal.
+
+
     Examples
     --------
     >>> import numpy as np
@@ -575,12 +618,27 @@ def ilr_inv(mat, basis=None, check=True):
     >>> ilr_inv(x)
     array([ 0.34180297,  0.29672718,  0.22054469,  0.14092516])
 
+    Notes
+    -----
+    If the `basis` parameter is specified, it is expected to be a basis in the
+    Aitchison simplex.  If there are `D-1` elements specified in `mat`, then
+    the dimensions of the basis needs be `D-1 x D`, where rows represent
+    basis vectors, and the columns represent proportions.
     """
 
     if basis is None:
         basis = _gram_schmidt_basis(mat.shape[-1] + 1)
-    elif check:
-        _check_orthogonality(basis)
+    else:
+        if len(basis.shape) != 2:
+            raise ValueError("Basis needs to be a 2D matrix, "
+                             "not a %dD matrix." %
+                             (len(basis.shape)))
+        if check:
+            _check_orthogonality(basis)
+        # this is necessary, since the clr function
+        # performs np.squeeze()
+        basis = np.atleast_2d(clr(basis))
+
     return clr_inv(np.dot(mat, basis))
 
 
@@ -620,15 +678,16 @@ def ancom(table, grouping,
           alpha=0.05,
           tau=0.02,
           theta=0.1,
-          multiple_comparisons_correction=None,
-          significance_test=None):
+          multiple_comparisons_correction='holm-bonferroni',
+          significance_test=None,
+          percentiles=(0.0, 25.0, 50.0, 75.0, 100.0)):
     r""" Performs a differential abundance test using ANCOM.
 
     This is done by calculating pairwise log ratios between all features
     and performing a significance test to determine if there is a significant
     difference in feature ratios with respect to the variable of interest.
 
-    In an experiment with only two treatments, this test tests the following
+    In an experiment with only two treatments, this tests the following
     hypothesis for feature :math:`i`
 
     .. math::
@@ -673,6 +732,10 @@ def ancom(table, grouping,
         classes.  This function must be able to accept at least two 1D
         array_like arguments of floats and returns a test statistic and a
         p-value. By default ``scipy.stats.f_oneway`` is used.
+    percentiles : iterable of floats, optional
+        Percentile abundances to return for each feature in each group. By
+        default, will return the minimum, 25th percentile, median, 75th
+        percentile, and maximum abundances for each feature in each group.
 
     Returns
     -------
@@ -683,7 +746,15 @@ def ancom(table, grouping,
         `"W"` is the W-statistic, or number of features that a single feature
         is tested to be significantly different against.
 
-        `"reject"` indicates if feature is significantly different or not.
+        `"Reject null hypothesis"` indicates if feature is differentially
+        abundant across groups (`True`) or not (`False`).
+
+    pd.DataFrame
+        A table of features and their percentile abundances in each group. If
+        ``percentiles`` is empty, this will be an empty ``pd.DataFrame``. The
+        rows in this object will be features, and the columns will be a
+        multi-index where the first index is the percentile, and the second
+        index is the group.
 
     See Also
     --------
@@ -696,22 +767,24 @@ def ancom(table, grouping,
     Notes
     -----
     The developers of this method recommend the following significance tests
-    ([2]_, Supplementary File 1, top of page 11): the standard parametric
-    t-test (``scipy.stats.ttest_ind``) or one-way ANOVA
-    (``scipy.stats.f_oneway``) if the number of groups is greater
-    than 2, or non-parametric variants such as Wilcoxon rank sum
-    (``scipy.stats.wilcoxon``) or Kruskal-Wallis (``scipy.stats.kruskal``)
-    if the number of groups is greater than 2.  Because one-way ANOVA is
-    equivalent to the standard t-test when the number of groups is two,
-    we default to ``scipy.stats.f_oneway`` here, which can be used when
-    there are two or more groups.  Users should refer to the documentation
-    of these tests in SciPy to understand the assumptions made by each test.
+    ([2]_, Supplementary File 1, top of page 11): if there are 2 groups, use
+    the standard parametric t-test (``scipy.stats.ttest_ind``) or
+    non-parametric Wilcoxon rank sum test (``scipy.stats.wilcoxon``).
+    If there are more than 2 groups, use parametric one-way ANOVA
+    (``scipy.stats.f_oneway``) or nonparametric Kruskal-Wallis
+    (``scipy.stats.kruskal``). Because one-way ANOVA is equivalent
+    to the standard t-test when the number of groups is two, we default to
+    ``scipy.stats.f_oneway`` here, which can be used when there are two or
+    more groups.  Users should refer to the documentation of these tests in
+    SciPy to understand the assumptions made by each test.
 
     This method cannot handle any zero counts as input, since the logarithm
     of zero cannot be computed.  While this is an unsolved problem, many
-    studies have shown promising results by replacing the zeros with pseudo
-    counts. This can be also be done via the ``multiplicative_replacement``
-    method.
+    studies, including [2]_, have shown promising results by adding
+    pseudocounts to all values in the matrix. In [2]_, a pseudocount of 0.001
+    was used, though the authors note that a pseudocount of 1.0 may also be
+    useful. Zero counts can also be addressed using the
+    ``multiplicative_replacement`` method.
 
     References
     ----------
@@ -728,7 +801,8 @@ def ancom(table, grouping,
     >>> from skbio.stats.composition import ancom
     >>> import pandas as pd
 
-    Now let's load in a pd.DataFrame with 6 samples and 7 unknown bacteria:
+    Now let's load in a DataFrame with 6 samples and 7 features (e.g.,
+    these may be bacterial OTUs):
 
     >>> table = pd.DataFrame([[12, 11, 10, 10, 10, 10, 10],
     ...                       [9,  11, 12, 10, 10, 10, 10],
@@ -736,25 +810,28 @@ def ancom(table, grouping,
     ...                       [22, 21, 9,  10, 10, 10, 10],
     ...                       [20, 22, 10, 10, 13, 10, 10],
     ...                       [23, 21, 14, 10, 10, 10, 10]],
-    ...                      index=['s1','s2','s3','s4','s5','s6'],
-    ...                      columns=['b1','b2','b3','b4','b5','b6','b7'])
+    ...                      index=['s1', 's2', 's3', 's4', 's5', 's6'],
+    ...                      columns=['b1', 'b2', 'b3', 'b4', 'b5', 'b6',
+    ...                               'b7'])
 
-    Then create a grouping vector.  In this scenario, there
-    are only two classes, and suppose these classes correspond to the
-    treatment due to a drug and a control.  The first three samples
-    are controls and the last three samples are treatments.
+    Then create a grouping vector. In this example, there is a treatment group
+    and a placebo group.
 
-    >>> grouping = pd.Series([0, 0, 0, 1, 1, 1],
-    ...                      index=['s1','s2','s3','s4','s5','s6'])
+    >>> grouping = pd.Series(['treatment', 'treatment', 'treatment',
+    ...                       'placebo', 'placebo', 'placebo'],
+    ...                      index=['s1', 's2', 's3', 's4', 's5', 's6'])
 
-    Now run ``ancom`` and see if there are any features that have any
-    significant differences between the treatment and the control.
+    Now run ``ancom`` to determine if there are any features that are
+    significantly different in abundance between the treatment and the placebo
+    groups. The first DataFrame that is returned contains the ANCOM test
+    results, and the second contains the percentile abundance data for each
+    feature in each group.
 
-    >>> results = ancom(table, grouping)
-    >>> results['W']
+    >>> ancom_df, percentile_df = ancom(table, grouping)
+    >>> ancom_df['W']
     b1    0
     b2    4
-    b3    1
+    b3    0
     b4    1
     b5    1
     b6    0
@@ -764,10 +841,13 @@ def ancom(table, grouping,
     The W-statistic is the number of features that a single feature is tested
     to be significantly different against.  In this scenario, `b2` was detected
     to have significantly different abundances compared to four of the other
-    species. To summarize the results from the W-statistic, let's take a look
-    at the results from the hypothesis test:
+    features. To summarize the results from the W-statistic, let's take a look
+    at the results from the hypothesis test. The `Reject null hypothesis`
+    column in the table indicates whether the null hypothesis was rejected,
+    and that a feature was therefore observed to be differentially abundant
+    across the groups.
 
-    >>> results['reject']
+    >>> ancom_df['Reject null hypothesis']
     b1    False
     b2     True
     b3    False
@@ -775,13 +855,44 @@ def ancom(table, grouping,
     b5    False
     b6    False
     b7    False
-    Name: reject, dtype: bool
+    Name: Reject null hypothesis, dtype: bool
 
-    From this we can conclude that only `b2` was significantly
-    different between the treatment and the control.
+    From this we can conclude that only `b2` was significantly different in
+    abundance between the treatment and the placebo. We still don't know, for
+    example, in which group `b2` was more abundant. We therefore may next be
+    interested in comparing the abundance of `b2` across the two groups.
+    We can do that using the second DataFrame that was returned. Here we
+    compare the median (50th percentile) abundance of `b2` in the treatment and
+    placebo groups:
+
+    >>> percentile_df[50.0].loc['b2']
+    Group
+    placebo      21.0
+    treatment    11.0
+    Name: b2, dtype: float64
+
+    We can also look at a full five-number summary for ``b2`` in the treatment
+    and placebo groups:
+
+    >>> percentile_df.loc['b2'] # doctest: +NORMALIZE_WHITESPACE
+    Percentile  Group
+    0.0         placebo      21.0
+    25.0        placebo      21.0
+    50.0        placebo      21.0
+    75.0        placebo      21.5
+    100.0       placebo      22.0
+    0.0         treatment    11.0
+    25.0        treatment    11.0
+    50.0        treatment    11.0
+    75.0        treatment    11.0
+    100.0       treatment    11.0
+    Name: b2, dtype: float64
+
+    Taken together, these data tell us that `b2` is present in significantly
+    higher abundance in the placebo group samples than in the treatment group
+    samples.
 
     """
-
     if not isinstance(table, pd.DataFrame):
         raise TypeError('`table` must be a `pd.DataFrame`, '
                         'not %r.' % type(table).__name__)
@@ -791,7 +902,7 @@ def ancom(table, grouping,
 
     if np.any(table <= 0):
         raise ValueError('Cannot handle zeros or negative values in `table`. '
-                         'Use pseudo counts or ``multiplicative_replacement``.'
+                         'Use pseudocounts or ``multiplicative_replacement``.'
                          )
 
     if not 0 < alpha < 1:
@@ -815,8 +926,20 @@ def ancom(table, grouping,
     if (table.isnull()).any().any():
         raise ValueError('Cannot handle missing values in `table`.')
 
-    groups, _grouping = np.unique(grouping, return_inverse=True)
-    grouping = pd.Series(_grouping, index=grouping.index)
+    percentiles = list(percentiles)
+    for percentile in percentiles:
+        if not 0.0 <= percentile <= 100.0:
+            raise ValueError('Percentiles must be in the range [0, 100], %r '
+                             'was provided.' % percentile)
+
+    duplicates = skbio.util.find_duplicates(percentiles)
+    if duplicates:
+        formatted_duplicates = ', '.join(repr(e) for e in duplicates)
+        raise ValueError('Percentile values must be unique. The following'
+                         ' value(s) were duplicated: %s.' %
+                         formatted_duplicates)
+
+    groups = np.unique(grouping)
     num_groups = len(groups)
 
     if num_groups == len(grouping):
@@ -873,9 +996,27 @@ def ancom(table, grouping,
         else:
             nu = cutoff[4]
         reject = (W >= nu*n_feat)
-    labs = mat.columns
-    return pd.DataFrame({'W': pd.Series(W, index=labs),
-                         'reject': pd.Series(reject, index=labs)})
+
+    feat_ids = mat.columns
+    ancom_df = pd.DataFrame(
+        {'W': pd.Series(W, index=feat_ids),
+         'Reject null hypothesis': pd.Series(reject, index=feat_ids)})
+
+    if len(percentiles) == 0:
+        return ancom_df, pd.DataFrame()
+    else:
+        data = []
+        columns = []
+        for group in groups:
+            feat_dists = mat[cats == group]
+            for percentile in percentiles:
+                columns.append((percentile, group))
+                data.append(np.percentile(feat_dists, percentile, axis=0))
+        columns = pd.MultiIndex.from_tuples(columns,
+                                            names=['Percentile', 'Group'])
+        percentile_df = pd.DataFrame(
+            np.asarray(data).T, columns=columns, index=feat_ids)
+        return ancom_df, percentile_df
 
 
 def _holm_bonferroni(p):
@@ -974,6 +1115,7 @@ def _check_orthogonality(basis):
     basis: numpy.ndarray
         basis in the Aitchison simplex
     """
+    basis = np.atleast_2d(basis)
     if not np.allclose(inner(basis, basis), np.identity(len(basis)),
                        rtol=1e-4, atol=1e-6):
         raise ValueError("Aitchison basis is not orthonormal")
