@@ -6,9 +6,7 @@
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
-from six import string_types
-
+import itertools
 from copy import deepcopy
 
 import matplotlib.pyplot as plt
@@ -38,7 +36,6 @@ class DistanceMatrixError(DissimilarityMatrixError):
 class MissingIDError(DissimilarityMatrixError):
     """Error for ID lookup that doesn't exist in the dissimilarity matrix."""
 
-    @experimental(as_of="0.4.0")
     def __init__(self, missing_id):
         super(MissingIDError, self).__init__()
         self.args = ("The ID '%s' is not in the dissimilarity matrix." %
@@ -113,6 +110,57 @@ class DissimilarityMatrix(SkbioObject):
         self._ids = ids
         self._id_index = self._index_list(self._ids)
 
+    @classonlymethod
+    @experimental(as_of="0.5.1")
+    def from_iterable(cls, iterable, metric, key=None, keys=None):
+        """Create DissimilarityMatrix from an iterable given a metric.
+
+        Parameters
+        ----------
+        iterable : iterable
+            Iterable containing objects to compute pairwise dissimilarities on.
+        metric : callable
+            A function that takes two arguments and returns a float
+            representing the dissimilarity between the two arguments.
+        key : callable or metadata key, optional
+            A function that takes one argument and returns a string
+            representing the id of the element in the dissimilarity matrix.
+            Alternatively, a key to a `metadata` property if it exists for
+            each element in the `iterable`. If None, then default ids will be
+            used.
+        keys : iterable, optional
+            An iterable of the same length as `iterable`. Each element will be
+            used as the respective key.
+
+        Returns
+        -------
+        DissimilarityMatrix
+            The `metric` applied to all pairwise elements in the `iterable`.
+
+        Raises
+        ------
+        ValueError
+            If `key` and `keys` are both provided.
+
+        """
+        iterable = list(iterable)
+        if key is not None and keys is not None:
+            raise ValueError("Cannot use both `key` and `keys` at the same"
+                             " time.")
+
+        keys_ = None
+        if key is not None:
+            keys_ = [resolve_key(e, key) for e in iterable]
+        elif keys is not None:
+            keys_ = keys
+
+        dm = np.empty((len(iterable),) * 2)
+        for i, a in enumerate(iterable):
+            for j, b in enumerate(iterable):
+                dm[i, j] = metric(a, b)
+
+        return cls(dm, keys_)
+
     @property
     @experimental(as_of="0.4.0")
     def data(self):
@@ -432,10 +480,10 @@ class DissimilarityMatrix(SkbioObject):
         ...                      [2, 3, 0]], ids=['a', 'b', 'c'])
         >>> df = dm.to_data_frame()
         >>> df
-           a  b  c
-        a  0  1  2
-        b  1  0  3
-        c  2  3  0
+             a    b    c
+        a  0.0  1.0  2.0
+        b  1.0  0.0  3.0
+        c  2.0  3.0  0.0
 
         """
         return pd.DataFrame(data=self.data, index=self.ids, columns=self.ids)
@@ -589,7 +637,7 @@ class DissimilarityMatrix(SkbioObject):
         The lookup based on ID(s) is quick.
 
         """
-        if isinstance(index, string_types):
+        if isinstance(index, str):
             return self.data[self.index(index)]
         elif self._is_id_pair(index):
             return self.data[self.index(index[0]), self.index(index[1])]
@@ -627,9 +675,6 @@ class DissimilarityMatrix(SkbioObject):
         if data.dtype != np.double:
             raise DissimilarityMatrixError("Data must contain only floating "
                                            "point values.")
-        if np.trace(data) != 0:
-            raise DissimilarityMatrixError("Data must be hollow (i.e., the "
-                                           "diagonal can only contain zeros).")
         duplicates = find_duplicates(ids)
         if duplicates:
             formatted_duplicates = ', '.join(repr(e) for e in duplicates)
@@ -648,7 +693,7 @@ class DissimilarityMatrix(SkbioObject):
     def _is_id_pair(self, index):
         return (isinstance(index, tuple) and
                 len(index) == 2 and
-                all(map(lambda e: isinstance(e, string_types), index)))
+                all(map(lambda e: isinstance(e, str), index)))
 
 
 class DistanceMatrix(DissimilarityMatrix):
@@ -687,7 +732,8 @@ class DistanceMatrix(DissimilarityMatrix):
 
     @classonlymethod
     @experimental(as_of="0.4.1")
-    def from_iterable(cls, iterable, metric, key=None, keys=None):
+    def from_iterable(cls, iterable, metric, key=None, keys=None,
+                      validate=True):
         """Create DistanceMatrix from all pairs in an iterable given a metric.
 
         Parameters
@@ -706,24 +752,30 @@ class DistanceMatrix(DissimilarityMatrix):
         keys : iterable, optional
             An iterable of the same length as `iterable`. Each element will be
             used as the respective key.
+        validate : boolean, optional
+            If ``True``, all pairwise distances are computed, including upper
+            and lower triangles and the diagonal, and the resulting matrix is
+            validated for symmetry and hollowness. If ``False``, `metric` is
+            assumed to be hollow and symmetric and only the lower triangle
+            (excluding the diagonal) is computed. Pass ``validate=False`` if
+            you are sure `metric` is hollow and symmetric for improved
+            performance.
 
         Returns
         -------
         DistanceMatrix
-            The `metric` applied to all pairwise elements in the `iterable`.
+            The `metric` applied to pairwise elements in the `iterable`.
 
         Raises
         ------
         ValueError
             If `key` and `keys` are both provided.
 
-        Notes
-        -----
-        Symmetry and hollowness are assumed when calculating the distances via
-        `metric`. Therefore, distances are only computed for the strictly
-        upper/lower triangle.
-
         """
+        if validate:
+            return super(DistanceMatrix, cls).from_iterable(iterable, metric,
+                                                            key, keys)
+
         iterable = list(iterable)
         if key is not None and keys is not None:
             raise ValueError("Cannot use both `key` and `keys` at the same"
@@ -816,7 +868,62 @@ class DistanceMatrix(DissimilarityMatrix):
         super(DistanceMatrix, self)._validate(data, ids)
 
         if (data.T != data).any():
-            raise DistanceMatrixError("Data must be symmetric.")
+            raise DistanceMatrixError(
+                "Data must be symmetric and cannot contain NaNs.")
+
+        if np.trace(data) != 0:
+            raise DistanceMatrixError("Data must be hollow (i.e., the diagonal"
+                                      " can only contain zeros).")
+
+    @experimental(as_of="0.5.1")
+    def to_series(self):
+        """Create a ``pandas.Series`` from this ``DistanceMatrix``.
+
+        The series will contain distances in condensed form: only distances
+        from one matrix triangle are included, and the diagonal is excluded.
+        The series' index will be a ``pd.MultiIndex`` relating pairs of IDs to
+        distances. The pairs of IDs will be in row-major order with respect to
+        the upper matrix triangle.
+
+        To obtain all distances (i.e. both upper and lower matrix triangles and
+        the diagonal), use ``DistanceMatrix.to_data_frame``. To obtain *only*
+        the distances in condensed form (e.g. for use with SciPy), use
+        ``DistanceMatrix.condensed_form``.
+
+        Returns
+        -------
+        pd.Series
+            ``pd.Series`` with pairs of IDs on the index.
+
+        See Also
+        --------
+        to_data_frame
+        condensed_form
+        scipy.spatial.distance.squareform
+
+        Examples
+        --------
+        >>> from skbio import DistanceMatrix
+        >>> dm = DistanceMatrix([[0, 1, 2, 3],
+        ...                      [1, 0, 4, 5],
+        ...                      [2, 4, 0, 6],
+        ...                      [3, 5, 6, 0]], ids=['a', 'b', 'c', 'd'])
+        >>> dm.to_series()
+        a  b    1.0
+           c    2.0
+           d    3.0
+        b  c    4.0
+           d    5.0
+        c  d    6.0
+        dtype: float64
+
+        """
+        distances = self.condensed_form()
+        # `id_pairs` will not be interpreted as a `pd.MultiIndex` if it is an
+        # iterable returned by `itertools.combinations`.
+        id_pairs = list(itertools.combinations(self.ids, 2))
+        index = pd.Index(id_pairs, tupleize_cols=True)
+        return pd.Series(data=distances, index=index, dtype=float)
 
 
 @experimental(as_of="0.4.0")
