@@ -140,16 +140,11 @@ we need to be confident that we have not committed a type II error increases.
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
-from future.utils import viewitems
-from future.builtins import range
-
 import collections
 import copy
 
 import numpy as np
 import scipy.stats
-import six
 
 from skbio.util._decorator import experimental
 
@@ -261,7 +256,7 @@ def subsample_power(test, samples, draw_mode='ind', alpha_pwr=0.05, ratio=None,
     `scipy.stats.chisquare` to look for the difference in frequency between
     groups.
 
-    >>> from scipy.stats import chisquare, nanmean
+    >>> from scipy.stats import chisquare
     >>> test = lambda x: chisquare(np.array([x[i].sum() for i in
     ...     range(len(x))]))[1]
 
@@ -287,10 +282,10 @@ def subsample_power(test, samples, draw_mode='ind', alpha_pwr=0.05, ratio=None,
     ...                                   counts_interval=5)
     >>> counts
     array([ 5, 10, 15, 20, 25, 30, 35, 40, 45])
-    >>> nanmean(pwr_est, 0) # doctest: +NORMALIZE_WHITESPACE
+    >>> np.nanmean(pwr_est, axis=0) # doctest: +NORMALIZE_WHITESPACE
     array([ 0.056,  0.074,  0.226,  0.46 ,  0.61 ,  0.806,  0.952,  1.   ,
             1.   ])
-    >>> counts[nanmean(pwr_est, 0) > 0.8].min()
+    >>> counts[np.nanmean(pwr_est, axis=0) > 0.8].min()
     30
 
     So, we can estimate that we will see a significant difference in the
@@ -338,9 +333,9 @@ def subsample_power(test, samples, draw_mode='ind', alpha_pwr=0.05, ratio=None,
     ...                                     ratio=[2, 1])
     >>> counts2
     array([  5.,  10.,  15.,  20.,  25.,  30.])
-    >>> nanmean(pwr_est2, 0)
+    >>> np.nanmean(pwr_est2, axis=0)
     array([ 0.14 ,  0.272,  0.426,  0.646,  0.824,  0.996])
-    >>> counts2[nanmean(pwr_est2, 0) > 0.8].min()
+    >>> counts2[np.nanmean(pwr_est2, axis=0) > 0.8].min()
     25.0
 
     When we consider the number of samples per group needed in the power
@@ -610,7 +605,9 @@ def confidence_bound(vec, alpha=0.05, df=None, axis=None):
         df = num_counts - 1
 
     # Calculates the bound
-    bound = scipy.stats.nanstd(vec, axis=axis) / np.sqrt(num_counts - 1) * \
+    # In the conversion from scipy.stats.nanstd -> np.nanstd `ddof=1` had to be
+    # added to match the scipy default of `bias=False`.
+    bound = np.nanstd(vec, axis=axis, ddof=1) / np.sqrt(num_counts - 1) * \
         scipy.stats.t.ppf(1 - alpha / 2, df)
 
     return bound
@@ -726,7 +723,7 @@ def _get_min_size(meta, cat, control_cats, order, strict_match):
 def _check_nans(x, switch=False):
     r"""Returns False if x is a nan and True is x is a string or number
     """
-    if isinstance(x, six.string_types):
+    if isinstance(x, str):
         return True
     elif isinstance(x, (float, int)):
         return not np.isnan(x)
@@ -1010,7 +1007,7 @@ def _identify_sample_groups(meta, cat, control_cats, order, strict_match):
     ctrl_groups = meta.groupby(control_cats).groups
     # Identifies the samples that satisfy the control pairs. Keys are iterated
     # in sorted order so that results don't change with different dictionary
-    # ordering (especially apparent in Python 3).
+    # ordering.
     for g in sorted(ctrl_groups, key=lambda k: str(k)):
         ids = ctrl_groups[g]
         # If strict_match, Skips over data that has nans
@@ -1020,10 +1017,10 @@ def _identify_sample_groups(meta, cat, control_cats, order, strict_match):
         m_ids = meta.loc[ids].groupby(cat).groups
         # Checks if samples from the cat groups are represented in those
         # Samples
-        ids_vec = id_vecs = [m_ids[o] for o in order if o in
-                             m_ids]
+        id_vecs = [m_ids[o] for o in order if o in
+                   m_ids]
         # If all groups are represented, the index and results are retained
-        if len(ids_vec) == len(order):
+        if len(id_vecs) == len(order):
             min_vec = np.array([len(v) for v in id_vecs])
             loc_vec = np.arange(0, min_vec.min())
             meta_pairs[i1] = id_vecs
@@ -1076,7 +1073,7 @@ def _draw_paired_samples(meta_pairs, index, num_samps):
     subs = []
 
     # Draws the other groups
-    for set_, num_ in viewitems(collections.Counter(set_pos)):
+    for set_, num_ in collections.Counter(set_pos).items():
         r2 = [np.random.choice(col, num_, replace=False) for col in
               meta_pairs[set_]]
         subs.append(r2)
