@@ -6,10 +6,7 @@
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
-from six import StringIO
-from future.builtins import zip
-
+import io
 from operator import attrgetter
 from unittest import TestCase, main
 
@@ -228,7 +225,7 @@ class GradientTests(BaseTests):
                                              's7': np.array([7]),
                                              's8': np.array([8])},
                                             orient='index')
-        trajectory.sort(columns=0, inplace=True)
+        trajectory.sort_values(by=0, inplace=True)
         w_vector = pd.Series(np.array([1, 5, 8, 12, 45, 80, 85, 90]),
                              ['s1', 's2', 's3', 's4',
                               's5', 's6', 's7', 's8']).astype(np.float64)
@@ -242,7 +239,7 @@ class GradientTests(BaseTests):
                                       's8': np.array([20.3428571428])},
                                      orient='index')
         obs = _weight_by_vector(trajectory, w_vector)
-        assert_data_frame_almost_equal(obs.sort(axis=0), exp.sort(axis=0))
+        assert_data_frame_almost_equal(obs.sort_index(), exp.sort_index())
 
         trajectory = pd.DataFrame.from_dict({'s1': np.array([1]),
                                              's2': np.array([2]),
@@ -253,18 +250,22 @@ class GradientTests(BaseTests):
                                              's7': np.array([7]),
                                              's8': np.array([8])},
                                             orient='index')
-        trajectory.sort(columns=0, inplace=True)
+        trajectory.sort_values(by=0, inplace=True)
         w_vector = pd.Series(np.array([1, 2, 3, 4, 5, 6, 7, 8]),
                              ['s1', 's2', 's3', 's4',
                               's5', 's6', 's7', 's8']).astype(np.float64)
-        exp = pd.DataFrame.from_dict({'s1': np.array([1]), 's2': np.array([2]),
-                                      's3': np.array([3]), 's4': np.array([4]),
-                                      's5': np.array([5]), 's6': np.array([6]),
-                                      's7': np.array([7]), 's8': np.array([8])
+        exp = pd.DataFrame.from_dict({'s1': np.array([1.0]),
+                                      's2': np.array([2.0]),
+                                      's3': np.array([3.0]),
+                                      's4': np.array([4.0]),
+                                      's5': np.array([5.0]),
+                                      's6': np.array([6.0]),
+                                      's7': np.array([7.0]),
+                                      's8': np.array([8.0])
                                       },
                                      orient='index')
         obs = _weight_by_vector(trajectory, w_vector)
-        assert_data_frame_almost_equal(obs.sort(axis=0), exp.sort(axis=0))
+        assert_data_frame_almost_equal(obs.sort_index(), exp.sort_index())
 
         trajectory = pd.DataFrame.from_dict({'s2': np.array([2]),
                                              's3': np.array([3]),
@@ -272,21 +273,23 @@ class GradientTests(BaseTests):
                                              's5': np.array([5]),
                                              's6': np.array([6])},
                                             orient='index')
-        trajectory.sort(columns=0, inplace=True)
+        trajectory.sort_values(by=0, inplace=True)
         w_vector = pd.Series(np.array([25, 30, 35, 40, 45]),
                              ['s2', 's3', 's4', 's5', 's6']).astype(np.float64)
-        exp = pd.DataFrame.from_dict({'s2': np.array([2]), 's3': np.array([3]),
-                                      's4': np.array([4]), 's5': np.array([5]),
-                                      's6': np.array([6])}, orient='index')
+        exp = pd.DataFrame.from_dict({'s2': np.array([2.0]),
+                                      's3': np.array([3.0]),
+                                      's4': np.array([4.0]),
+                                      's5': np.array([5.0]),
+                                      's6': np.array([6.0])}, orient='index')
         obs = _weight_by_vector(trajectory, w_vector)
-        assert_data_frame_almost_equal(obs.sort(axis=0), exp.sort(axis=0))
+        assert_data_frame_almost_equal(obs.sort_index(), exp.sort_index())
 
         trajectory = pd.DataFrame.from_dict({'s1': np.array([1, 2, 3]),
                                              's2': np.array([2, 3, 4]),
                                              's3': np.array([5, 6, 7]),
                                              's4': np.array([8, 9, 10])},
                                             orient='index')
-        trajectory.sort(columns=0, inplace=True)
+        trajectory.sort_values(by=0, inplace=True)
         w_vector = pd.Series(np.array([1, 2, 3, 4]),
                              ['s1', 's2', 's3', 's4']).astype(np.float64)
         exp = pd.DataFrame.from_dict({'s1': np.array([1, 2, 3]),
@@ -295,7 +298,7 @@ class GradientTests(BaseTests):
                                       's4': np.array([8, 9, 10])},
                                      orient='index').astype(np.float64)
         obs = _weight_by_vector(trajectory, w_vector)
-        assert_data_frame_almost_equal(obs.sort(axis=0), exp.sort(axis=0))
+        assert_data_frame_almost_equal(obs.sort_index(), exp.sort_index())
 
         sample_ids = ['PC.356', 'PC.481', 'PC.355', 'PC.593', 'PC.354']
         trajectory = pd.DataFrame.from_dict({'PC.356': np.array([5.65948525,
@@ -334,7 +337,7 @@ class GradientTests(BaseTests):
                                       }, orient='index')
         obs = _weight_by_vector(trajectory.ix[sample_ids],
                                 w_vector[sample_ids])
-        assert_data_frame_almost_equal(obs.sort(axis=0), exp.sort(axis=0))
+        assert_data_frame_almost_equal(obs.sort_index(), exp.sort_index())
 
     def test_weight_by_vector_single_element(self):
         trajectory = pd.DataFrame.from_dict({'s1': np.array([42])},
@@ -401,8 +404,8 @@ class GroupResultsTests(BaseTests):
         raw_paths = ['gr_wo_msg_raw', 'gr_w_msg_raw']
 
         for gr, out_fp, raw_fp in zip(self.groups, out_paths, raw_paths):
-            obs_out_f = StringIO()
-            obs_raw_f = StringIO()
+            obs_out_f = io.StringIO()
+            obs_raw_f = io.StringIO()
             gr.to_files(obs_out_f, obs_raw_f)
             obs_out = obs_out_f.getvalue()
             obs_raw = obs_raw_f.getvalue()
@@ -425,8 +428,8 @@ class CategoryResultsTests(BaseTests):
         raw_paths = ['cr_no_data_raw', 'cr_data_raw']
 
         for cat, out_fp, raw_fp in zip(self.categories, out_paths, raw_paths):
-            obs_out_f = StringIO()
-            obs_raw_f = StringIO()
+            obs_out_f = io.StringIO()
+            obs_raw_f = io.StringIO()
             cat.to_files(obs_out_f, obs_raw_f)
             obs_out = obs_out_f.getvalue()
             obs_raw = obs_raw_f.getvalue()
@@ -449,8 +452,8 @@ class GradientANOVAResultsTests(BaseTests):
         raw_paths = ['vr_raw']
 
         for vr, out_fp, raw_fp in zip(self.vec_results, out_paths, raw_paths):
-            obs_out_f = StringIO()
-            obs_raw_f = StringIO()
+            obs_out_f = io.StringIO()
+            obs_raw_f = io.StringIO()
             vr.to_files(obs_out_f, obs_raw_f)
             obs_out = obs_out_f.getvalue()
             obs_raw = obs_raw_f.getvalue()
@@ -578,20 +581,20 @@ class GradientANOVATests(BaseTests):
         # Takes a subset from metadata_map
         bv = GradientANOVA(subset_coords, self.prop_expl, self.metadata_map)
         assert_data_frame_almost_equal(
-            bv._coords.sort(axis=0),
-            subset_coords.sort(axis=0))
+            bv._coords.sort_index(),
+            subset_coords.sort_index())
         assert_data_frame_almost_equal(
-            bv._metadata_map.sort(axis=0),
-            subset_metadata_map.sort(axis=0))
+            bv._metadata_map.sort_index(),
+            subset_metadata_map.sort_index())
 
         # Takes a subset from coords
         bv = GradientANOVA(self.coords, self.prop_expl, subset_metadata_map)
         assert_data_frame_almost_equal(
-            bv._coords.sort(axis=0),
-            subset_coords.sort(axis=0))
+            bv._coords.sort_index(),
+            subset_coords.sort_index())
         assert_data_frame_almost_equal(
-            bv._metadata_map.sort(axis=0),
-            subset_metadata_map.sort(axis=0))
+            bv._metadata_map.sort_index(),
+            subset_metadata_map.sort_index())
 
         # Takes a subset from metadata_map and coords at the same time
         coord_data = {
@@ -625,8 +628,8 @@ class GradientANOVATests(BaseTests):
                                  -0.0301637746424])},
             orient='index')
         assert_data_frame_almost_equal(
-            bv._coords.sort(axis=0),
-            exp_coords.sort(axis=0))
+            bv._coords.sort_index(),
+            exp_coords.sort_index())
         exp_metadata_map = pd.DataFrame.from_dict(
             {'PC.355': {'Treatment': 'Control',
                         'DOB': '20061218',
@@ -634,8 +637,8 @@ class GradientANOVATests(BaseTests):
                         'Description': 'Control_mouse_I.D._355'}},
             orient='index')
         assert_data_frame_almost_equal(
-            bv._metadata_map.sort(axis=0),
-            exp_metadata_map.sort(axis=0))
+            bv._metadata_map.sort_index(),
+            exp_metadata_map.sort_index())
 
     def test_normalize_samples_error(self):
         """Raises an error if coords and metadata_map does not have samples in
