@@ -6,8 +6,6 @@
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
-
 import warnings
 from operator import or_, itemgetter
 from copy import deepcopy
@@ -17,8 +15,6 @@ from collections import defaultdict
 
 import numpy as np
 from scipy.stats import pearsonr
-from future.builtins import zip
-import six
 
 from skbio._base import SkbioObject
 from skbio.stats.distance import DistanceMatrix
@@ -396,9 +392,24 @@ class TreeNode(SkbioObject):
             else:
                 child.length += node.length
 
+            if node.parent is None:
+                continue
+
             node.parent.append(child)
             node.parent.remove(node)
 
+        # if a single descendent from the root, the root adopts the childs
+        # properties. we can't "delete" the root as that would be deleting
+        # self.
+        if len(self.children) == 1:
+            node_to_copy = self.children[0]
+            efc = self._exclude_from_copy
+            for key in node_to_copy.__dict__:
+                if key not in efc:
+                    self.__dict__[key] = deepcopy(node_to_copy.__dict__[key])
+            self.remove(node_to_copy)
+            self.extend(node_to_copy.children)
+
     @experimental(as_of="0.4.0")
     def shear(self, names):
         """Lop off tips until the tree just has the desired tip names.
@@ -442,10 +453,19 @@ class TreeNode(SkbioObject):
         if not ids.issubset(all_tips):
             raise ValueError("ids are not a subset of the tree.")
 
-        while len(list(tcopy.tips())) != len(ids):
-            for n in list(tcopy.tips()):
-                if n.name not in ids:
-                    n.parent.remove(n)
+        marked = set()
+        for tip in tcopy.tips():
+            if tip.name in ids:
+                marked.add(tip)
+                for anc in tip.ancestors():
+                    if anc in marked:
+                        break
+                    else:
+                        marked.add(anc)
+
+        for node in list(tcopy.traverse()):
+            if node not in marked:
+                node.parent.remove(node)
 
         tcopy.prune()
 
@@ -795,7 +815,7 @@ class TreeNode(SkbioObject):
         <BLANKLINE>
 
         """
-        if isinstance(node, six.string_types):
+        if isinstance(node, str):
             node = self.find(node)
 
         if not node.children:
@@ -1176,7 +1196,7 @@ class TreeNode(SkbioObject):
         if not self.children:
             if include_self:
                 yield self
-            raise StopIteration
+            return
         child_index_stack = [0]
         curr = self
         curr_children = self.children
@@ -1766,7 +1786,8 @@ class TreeNode(SkbioObject):
         Raises
         ------
         ValueError
-            If no tips could be found in the tree
+            If no tips could be found in the tree, or if not all tips were
+            found.
 
         Examples
         --------
@@ -1883,13 +1904,13 @@ class TreeNode(SkbioObject):
                 if name in cur_node._lookup:
                     cur_node = cur_node._lookup[name]
                 else:
-                    new_node = TreeNode(name=name)
+                    new_node = cls(name=name)
                     new_node._lookup = {}
                     cur_node._lookup[name] = new_node
                     cur_node.append(new_node)
                     cur_node = new_node
 
-            cur_node.append(TreeNode(name=id_))
+            cur_node.append(cls(name=id_))
 
         # scrub the lookups
         for node in root.non_tips(include_self=True):
@@ -1942,13 +1963,13 @@ class TreeNode(SkbioObject):
         tip_width = len(id_list)
         cluster_count = len(linkage_matrix)
         lookup_len = cluster_count + tip_width
-        node_lookup = np.empty(lookup_len, dtype=TreeNode)
+        node_lookup = np.empty(lookup_len, dtype=cls)
 
         for i, name in enumerate(id_list):
-            node_lookup[i] = TreeNode(name=name)
+            node_lookup[i] = cls(name=name)
 
         for i in range(tip_width, lookup_len):
-            node_lookup[i] = TreeNode()
+            node_lookup[i] = cls()
 
         newest_cluster_index = cluster_count + 1
         for link in linkage_matrix:
@@ -2305,12 +2326,20 @@ class TreeNode(SkbioObject):
         if self is other:
             return 0.0
 
-        root = self.root()
-        lca = root.lowest_common_ancestor([self, other])
-        accum = self.accumulate_to_ancestor(lca)
-        accum += other.accumulate_to_ancestor(lca)
+        self_ancestors = [self] + list(self.ancestors())
+        other_ancestors = [other] + list(other.ancestors())
 
-        return accum
+        if self in other_ancestors:
+            return other.accumulate_to_ancestor(self)
+        elif other in self_ancestors:
+            return self.accumulate_to_ancestor(other)
+        else:
+            root = self.root()
+            lca = root.lowest_common_ancestor([self, other])
+            accum = self.accumulate_to_ancestor(lca)
+            accum += other.accumulate_to_ancestor(lca)
+
+            return accum
 
     def _set_max_distance(self):
         """Propagate tip distance information up the tree
@@ -2716,6 +2745,71 @@ class TreeNode(SkbioObject):
 
         return dist_f(self_matrix, other_matrix)
 
+    @experimental(as_of="0.4.2")
+    def bifurcate(self, insert_length=None):
+        r"""Reorders the tree into a bifurcating tree.
+
+        All nodes that have more than 2 children will
+        have additional intermediate nodes inserted to ensure that
+        every node has only 2 children.
+
+        Parameters
+        ----------
+        insert_length : int, optional
+            The branch length assigned to all inserted nodes.
+
+        See Also
+        --------
+        prune
+
+        Notes
+        -----
+        Any nodes that have a single child can be collapsed using the
+        prune method to create strictly bifurcating trees.
+
+        Examples
+        --------
+        >>> from skbio import TreeNode
+        >>> tree = TreeNode.read(["((a,b,g,h)c,(d,e)f)root;"])
+        >>> print(tree.ascii_art())
+                            /-a
+                           |
+                           |--b
+                  /c-------|
+                 |         |--g
+                 |         |
+        -root----|          \-h
+                 |
+                 |          /-d
+                  \f-------|
+                            \-e
+        >>> tree.bifurcate()
+        >>> print(tree.ascii_art())
+                            /-h
+                  /c-------|
+                 |         |          /-g
+                 |          \--------|
+                 |                   |          /-a
+        -root----|                    \--------|
+                 |                              \-b
+                 |
+                 |          /-d
+                  \f-------|
+                            \-e
+        """
+        for n in self.traverse(include_self=True):
+            if len(n.children) > 2:
+                stack = n.children
+                while len(stack) > 2:
+                    ind = stack.pop()
+                    intermediate = self.__class__()
+                    intermediate.length = insert_length
+                    intermediate.extend(stack)
+                    n.append(intermediate)
+                    for k in stack:
+                        n.remove(k)
+                    n.extend([ind, intermediate])
+
     @experimental(as_of="0.4.0")
     def index_tree(self):
         """Index a tree for rapid lookups within a tree array
