@@ -6,8 +6,7 @@
 # The full license is in the file COPYING.txt, distributed with this software.
 # ----------------------------------------------------------------------------
 
-from __future__ import absolute_import, division, print_function
-
+import io
 from unittest import TestCase, main
 from collections import defaultdict
 
@@ -15,18 +14,21 @@ import numpy as np
 import numpy.testing as npt
 from scipy.stats import pearsonr
 
-from skbio.io._fileobject import StringIO
 from skbio import DistanceMatrix, TreeNode
 from skbio.tree import (DuplicateNodeError, NoLengthError,
                         TreeError, MissingNodeError, NoParentError)
 from skbio.util import RepresentationWarning
 
 
+class TreeNodeSubclass(TreeNode):
+    pass
+
+
 class TreeTests(TestCase):
 
     def setUp(self):
         """Prep the self"""
-        self.simple_t = TreeNode.read(StringIO(u"((a,b)i1,(c,d)i2)root;"))
+        self.simple_t = TreeNode.read(io.StringIO("((a,b)i1,(c,d)i2)root;"))
         nodes = dict([(x, TreeNode(x)) for x in 'abcdefgh'])
         nodes['a'].append(nodes['b'])
         nodes['b'].append(nodes['c'])
@@ -35,7 +37,6 @@ class TreeTests(TestCase):
         nodes['c'].append(nodes['f'])
         nodes['f'].append(nodes['g'])
         nodes['a'].append(nodes['h'])
-        self.TreeNode = nodes
         self.TreeRoot = nodes['a']
 
         def rev_f(items):
@@ -48,9 +49,19 @@ class TreeTests(TestCase):
 
         self.rev_f = rev_f
         self.rotate_f = rotate_f
-        self.complex_tree = TreeNode.read(StringIO(u"(((a,b)int1,(x,y,(w,z)int"
-                                                   "2,(c,d)int3)int4),(e,f)int"
-                                                   "5);"))
+        self.complex_tree = TreeNode.read(io.StringIO(
+            "(((a,b)int1,(x,y,(w,z)int2,(c,d)int3)int4),(e,f)int5);"))
+
+    def test_bug_issue_1416(self):
+        tree = TreeNode.read(['(((a,b,f,g),c),d);'])
+        new_tree = tree.shear(['a', 'b', 'c', 'f'])
+
+        exp = {'a', 'b', 'c', 'f'}
+        obs = {n.name for n in new_tree.tips()}
+
+        self.assertEqual(obs, exp)
+        self.assertEqual(id(new_tree), id(new_tree.children[0].parent))
+        self.assertEqual(id(new_tree), id(new_tree.children[1].parent))
 
     def test_observed_node_counts(self):
         """returns observed nodes counts given vector of otu observation counts
@@ -171,7 +182,7 @@ class TreeTests(TestCase):
 
     def test_append(self):
         """Append a node to a tree"""
-        second_tree = TreeNode.read(StringIO(u"(x,y)z;"))
+        second_tree = TreeNode.read(io.StringIO("(x,y)z;"))
         self.simple_t.append(second_tree)
 
         self.assertEqual(self.simple_t.children[0].name, 'i1')
@@ -184,10 +195,10 @@ class TreeTests(TestCase):
 
     def test_extend(self):
         """Extend a few nodes"""
-        second_tree = TreeNode.read(StringIO(u"(x1,y1)z1;"))
-        third_tree = TreeNode.read(StringIO(u"(x2,y2)z2;"))
-        first_tree = TreeNode.read(StringIO(u"(x1,y1)z1;"))
-        fourth_tree = TreeNode.read(StringIO(u"(x2,y2)z2;"))
+        second_tree = TreeNode.read(io.StringIO("(x1,y1)z1;"))
+        third_tree = TreeNode.read(io.StringIO("(x2,y2)z2;"))
+        first_tree = TreeNode.read(io.StringIO("(x1,y1)z1;"))
+        fourth_tree = TreeNode.read(io.StringIO("(x2,y2)z2;"))
         self.simple_t.extend([second_tree, third_tree])
 
         first_tree.extend(fourth_tree.children)
@@ -243,8 +254,8 @@ class TreeTests(TestCase):
 
     def test_pop(self):
         """Pop off a node"""
-        second_tree = TreeNode.read(StringIO(u"(x1,y1)z1;"))
-        third_tree = TreeNode.read(StringIO(u"(x2,y2)z2;"))
+        second_tree = TreeNode.read(io.StringIO("(x1,y1)z1;"))
+        third_tree = TreeNode.read(io.StringIO("(x2,y2)z2;"))
         self.simple_t.extend([second_tree, third_tree])
 
         i1 = self.simple_t.pop(0)
@@ -303,6 +314,27 @@ class TreeTests(TestCase):
         self.assertEqual(len(n.children), 2)
         self.assertNotIn(n, self.simple_t.children)
 
+    def test_shear_prune_parent_dropped(self):
+        bugtree = "((a,b),((c,d),(e,f)));"
+        to_keep = ['c', 'd']
+        exp = "(c,d);\n"
+        obs = str(TreeNode.read(io.StringIO(bugtree)).shear(to_keep))
+        self.assertEqual(obs, exp)
+
+    def test_prune_nested_single_descendent(self):
+        bugtree = "(((a,b)));"
+        exp = "(a,b);\n"
+        t = TreeNode.read(io.StringIO(bugtree))
+        t.prune()
+        obs = str(t)
+        self.assertEqual(obs, exp)
+
+    def test_prune_root_single_desc(self):
+        t = TreeNode.read(["((a,b)c)extra;"])
+        exp = "(a,b)c;\n"
+        t.prune()
+        self.assertEqual(str(t), exp)
+
     def test_prune(self):
         """Collapse single descendent nodes"""
         # check the identity case
@@ -397,7 +429,7 @@ class TreeTests(TestCase):
         self.assertEqual(root._non_tip_cache, {})
 
     def test_invalidate_attr_caches(self):
-        tree = TreeNode.read(StringIO(u"((a,b,(c,d)e)f,(g,h)i)root;"))
+        tree = TreeNode.read(io.StringIO("((a,b,(c,d)e)f,(g,h)i)root;"))
 
         def f(n):
             return [n.name] if n.is_tip() else []
@@ -409,10 +441,10 @@ class TreeTests(TestCase):
 
     def test_create_caches_duplicate_tip_names(self):
         with self.assertRaises(DuplicateNodeError):
-            TreeNode.read(StringIO(u'(a, a);')).create_caches()
+            TreeNode.read(io.StringIO('(a, a);')).create_caches()
 
     def test_find_all(self):
-        t = TreeNode.read(StringIO(u"((a,b)c,((d,e)c)c,(f,(g,h)c)a)root;"))
+        t = TreeNode.read(io.StringIO("((a,b)c,((d,e)c)c,(f,(g,h)c)a)root;"))
         exp = [t.children[0],
                t.children[1].children[0],
                t.children[1],
@@ -438,7 +470,7 @@ class TreeTests(TestCase):
 
     def test_find(self):
         """Find a node in a tree"""
-        t = TreeNode.read(StringIO(u"((a,b)c,(d,e)f);"))
+        t = TreeNode.read(io.StringIO("((a,b)c,(d,e)f);"))
         exp = t.children[0]
         obs = t.find('c')
         self.assertEqual(obs, exp)
@@ -452,7 +484,7 @@ class TreeTests(TestCase):
 
     def test_find_cache_bug(self):
         """First implementation did not force the cache to be at the root"""
-        t = TreeNode.read(StringIO(u"((a,b)c,(d,e)f,(g,h)f);"))
+        t = TreeNode.read(io.StringIO("((a,b)c,(d,e)f,(g,h)f);"))
         exp_tip_cache_keys = set(['a', 'b', 'd', 'e', 'g', 'h'])
         exp_non_tip_cache_keys = set(['c', 'f'])
         tip_a = t.children[0].children[0]
@@ -464,8 +496,8 @@ class TreeTests(TestCase):
 
     def test_find_by_id(self):
         """Find a node by id"""
-        t1 = TreeNode.read(StringIO(u"((,),(,,));"))
-        t2 = TreeNode.read(StringIO(u"((,),(,,));"))
+        t1 = TreeNode.read(io.StringIO("((,),(,,));"))
+        t2 = TreeNode.read(io.StringIO("((,),(,,));"))
 
         exp = t1.children[1]
         obs = t1.find_by_id(6)  # right inner node with 3 children
@@ -480,7 +512,7 @@ class TreeTests(TestCase):
 
     def test_find_by_func(self):
         """Find nodes by a function"""
-        t = TreeNode.read(StringIO(u"((a,b)c,(d,e)f);"))
+        t = TreeNode.read(io.StringIO("((a,b)c,(d,e)f);"))
 
         def func(x):
             return x.parent == t.find('c')
@@ -526,7 +558,7 @@ class TreeTests(TestCase):
     def test_ascii_art(self):
         """Make some ascii trees"""
         # unlabeled internal node
-        tr = TreeNode.read(StringIO(u"(B:0.2,(C:0.3,D:0.4):0.6)F;"))
+        tr = TreeNode.read(io.StringIO("(B:0.2,(C:0.3,D:0.4):0.6)F;"))
         obs = tr.ascii_art(show_internal=True, compact=False)
         exp = "          /-B\n-F-------|\n         |          /-C\n         "\
               " \\--------|\n                    \\-D"
@@ -540,13 +572,13 @@ class TreeTests(TestCase):
         self.assertEqual(obs, exp)
 
     def test_ascii_art_three_children(self):
-        obs = TreeNode.read(StringIO(u'(a,(b,c,d));')).ascii_art()
+        obs = TreeNode.read(io.StringIO('(a,(b,c,d));')).ascii_art()
         self.assertEqual(obs, exp_ascii_art_three_children)
 
     def test_accumulate_to_ancestor(self):
         """Get the distance from a node to its ancestor"""
-        t = TreeNode.read(StringIO(
-            u"((a:0.1,b:0.2)c:0.3,(d:0.4,e)f:0.5)root;"))
+        t = TreeNode.read(io.StringIO(
+            "((a:0.1,b:0.2)c:0.3,(d:0.4,e)f:0.5)root;"))
         a = t.find('a')
         b = t.find('b')
         exp_to_root = 0.1 + 0.3
@@ -556,10 +588,16 @@ class TreeTests(TestCase):
         with self.assertRaises(NoParentError):
             a.accumulate_to_ancestor(b)
 
+    def test_distance_nontip(self):
+        # example derived from issue #807, credit @wwood
+        tstr = "((A:1.0,B:2.0)'g__genus1':3.0)root;"
+        tree = TreeNode.read(io.StringIO(tstr))
+        self.assertEqual(tree.find('A').distance(tree.find('g__genus1')), 1.0)
+
     def test_distance(self):
         """Get the distance between two nodes"""
-        t = TreeNode.read(StringIO(
-            u"((a:0.1,b:0.2)c:0.3,(d:0.4,e)f:0.5)root;"))
+        t = TreeNode.read(io.StringIO(
+            "((a:0.1,b:0.2)c:0.3,(d:0.4,e)f:0.5)root;"))
         tips = sorted([n for n in t.tips()], key=lambda x: x.name)
 
         npt.assert_almost_equal(tips[0].distance(tips[0]), 0.0)
@@ -582,7 +620,7 @@ class TreeTests(TestCase):
 
     def test_lowest_common_ancestor(self):
         """TreeNode lowestCommonAncestor should return LCA for set of tips"""
-        t1 = TreeNode.read(StringIO(u"((a,(b,c)d)e,f,(g,h)i)j;"))
+        t1 = TreeNode.read(io.StringIO("((a,(b,c)d)e,f,(g,h)i)j;"))
         t2 = t1.copy()
         t3 = t1.copy()
         t4 = t1.copy()
@@ -618,16 +656,16 @@ class TreeTests(TestCase):
 
     def test_get_max_distance(self):
         """get_max_distance should get max tip distance across tree"""
-        tree = TreeNode.read(StringIO(
-            u"((a:0.1,b:0.2)c:0.3,(d:0.4,e:0.5)f:0.6)root;"))
+        tree = TreeNode.read(io.StringIO(
+            "((a:0.1,b:0.2)c:0.3,(d:0.4,e:0.5)f:0.6)root;"))
         dist, nodes = tree.get_max_distance()
         npt.assert_almost_equal(dist, 1.6)
         self.assertEqual(sorted([n.name for n in nodes]), ['b', 'e'])
 
     def test_set_max_distance(self):
         """set_max_distance sets MaxDistTips across tree"""
-        tree = TreeNode.read(StringIO(
-            u"((a:0.1,b:0.2)c:0.3,(d:0.4,e:0.5)f:0.6)root;"))
+        tree = TreeNode.read(io.StringIO(
+            "((a:0.1,b:0.2)c:0.3,(d:0.4,e:0.5)f:0.6)root;"))
         tree._set_max_distance()
         tip_a, tip_b = tree.MaxDistTips
         self.assertEqual(tip_a[0] + tip_b[0], 1.6)
@@ -635,7 +673,7 @@ class TreeTests(TestCase):
 
     def test_set_max_distance_tie_bug(self):
         """Corresponds to #1077"""
-        s = StringIO("((a:1,b:1)c:2,(d:3,e:4)f:5)root;")
+        s = io.StringIO("((a:1,b:1)c:2,(d:3,e:4)f:5)root;")
         t = TreeNode.read(s)
 
         exp = ((3.0, t.find('a')), (9.0, t.find('e')))
@@ -651,7 +689,7 @@ class TreeTests(TestCase):
 
     def test_set_max_distance_inplace_modification_bug(self):
         """Corresponds to #1223"""
-        s = StringIO("((a:1,b:1)c:2,(d:3,e:4)f:5)root;")
+        s = io.StringIO("((a:1,b:1)c:2,(d:3,e:4)f:5)root;")
         t = TreeNode.read(s)
 
         exp = [((0.0, t.find('a')), (0.0, t.find('a'))),
@@ -668,14 +706,14 @@ class TreeTests(TestCase):
 
     def test_shear(self):
         """Shear the nodes"""
-        t = TreeNode.read(StringIO(u'((H:1,G:1):2,(R:0.5,M:0.7):3);'))
+        t = TreeNode.read(io.StringIO('((H:1,G:1):2,(R:0.5,M:0.7):3);'))
         obs = str(t.shear(['G', 'M']))
         exp = '(G:3.0,M:3.7);\n'
         self.assertEqual(obs, exp)
 
     def test_compare_tip_distances(self):
-        t = TreeNode.read(StringIO(u'((H:1,G:1):2,(R:0.5,M:0.7):3);'))
-        t2 = TreeNode.read(StringIO(u'(((H:1,G:1,O:1):2,R:3):1,X:4);'))
+        t = TreeNode.read(io.StringIO('((H:1,G:1):2,(R:0.5,M:0.7):3);'))
+        t2 = TreeNode.read(io.StringIO('(((H:1,G:1,O:1):2,R:3):1,X:4);'))
         obs = t.compare_tip_distances(t2)
         # note: common taxa are H, G, R (only)
         m1 = np.array([[0, 2, 6.5], [2, 0, 6.5], [6.5, 6.5, 0]])
@@ -684,8 +722,8 @@ class TreeTests(TestCase):
         self.assertAlmostEqual(obs, (1 - r) / 2)
 
     def test_compare_tip_distances_sample(self):
-        t = TreeNode.read(StringIO(u'((H:1,G:1):2,(R:0.5,M:0.7):3);'))
-        t2 = TreeNode.read(StringIO(u'(((H:1,G:1,O:1):2,R:3):1,X:4);'))
+        t = TreeNode.read(io.StringIO('((H:1,G:1):2,(R:0.5,M:0.7):3);'))
+        t2 = TreeNode.read(io.StringIO('(((H:1,G:1,O:1):2,R:3):1,X:4);'))
         obs = t.compare_tip_distances(t2, sample=3, shuffle_f=sorted)
         # note: common taxa are H, G, R (only)
         m1 = np.array([[0, 2, 6.5], [2, 0, 6.5], [6.5, 6.5, 0]])
@@ -694,29 +732,29 @@ class TreeTests(TestCase):
         self.assertAlmostEqual(obs, (1 - r) / 2)
 
         # 4 common taxa, still picking H, G, R
-        s = u'((H:1,G:1):2,(R:0.5,M:0.7,Q:5):3);'
-        t = TreeNode.read(StringIO(s))
-        s3 = u'(((H:1,G:1,O:1):2,R:3,Q:10):1,X:4);'
-        t3 = TreeNode.read(StringIO(s3))
+        s = '((H:1,G:1):2,(R:0.5,M:0.7,Q:5):3);'
+        t = TreeNode.read(io.StringIO(s))
+        s3 = '(((H:1,G:1,O:1):2,R:3,Q:10):1,X:4);'
+        t3 = TreeNode.read(io.StringIO(s3))
         obs = t.compare_tip_distances(t3, sample=3, shuffle_f=sorted)
 
     def test_compare_tip_distances_no_common_tips(self):
-        t = TreeNode.read(StringIO(u'((H:1,G:1):2,(R:0.5,M:0.7):3);'))
-        t2 = TreeNode.read(StringIO(u'(((Z:1,Y:1,X:1):2,W:3):1,V:4);'))
+        t = TreeNode.read(io.StringIO('((H:1,G:1):2,(R:0.5,M:0.7):3);'))
+        t2 = TreeNode.read(io.StringIO('(((Z:1,Y:1,X:1):2,W:3):1,V:4);'))
 
         with self.assertRaises(ValueError):
             t.compare_tip_distances(t2)
 
     def test_compare_tip_distances_single_common_tip(self):
-        t = TreeNode.read(StringIO(u'((H:1,G:1):2,(R:0.5,M:0.7):3);'))
-        t2 = TreeNode.read(StringIO(u'(((R:1,Y:1,X:1):2,W:3):1,V:4);'))
+        t = TreeNode.read(io.StringIO('((H:1,G:1):2,(R:0.5,M:0.7):3);'))
+        t2 = TreeNode.read(io.StringIO('(((R:1,Y:1,X:1):2,W:3):1,V:4);'))
 
         self.assertEqual(t.compare_tip_distances(t2), 1)
         self.assertEqual(t2.compare_tip_distances(t), 1)
 
     def test_tip_tip_distances_endpoints(self):
         """Test getting specifc tip distances  with tipToTipDistances"""
-        t = TreeNode.read(StringIO(u'((H:1,G:1):2,(R:0.5,M:0.7):3);'))
+        t = TreeNode.read(io.StringIO('((H:1,G:1):2,(R:0.5,M:0.7):3);'))
         nodes = [t.find('H'), t.find('G'), t.find('M')]
         names = ['H', 'G', 'M']
         exp = DistanceMatrix(np.array([[0, 2.0, 6.7],
@@ -730,13 +768,13 @@ class TreeTests(TestCase):
         self.assertEqual(obs, exp)
 
     def test_tip_tip_distances_non_tip_endpoints(self):
-        t = TreeNode.read(StringIO(u'((H:1,G:1)foo:2,(R:0.5,M:0.7):3);'))
+        t = TreeNode.read(io.StringIO('((H:1,G:1)foo:2,(R:0.5,M:0.7):3);'))
         with self.assertRaises(ValueError):
             t.tip_tip_distances(endpoints=['foo'])
 
     def test_tip_tip_distances_no_length(self):
-        t = TreeNode.read(StringIO(u"((a,b)c,(d,e)f);"))
-        exp_t = TreeNode.read(StringIO(u"((a:0,b:0)c:0,(d:0,e:0)f:0);"))
+        t = TreeNode.read(io.StringIO("((a,b)c,(d,e)f);"))
+        exp_t = TreeNode.read(io.StringIO("((a:0,b:0)c:0,(d:0,e:0)f:0);"))
         exp_t_dm = exp_t.tip_tip_distances()
 
         t_dm = npt.assert_warns(RepresentationWarning, t.tip_tip_distances)
@@ -746,8 +784,8 @@ class TreeTests(TestCase):
             self.assertIs(node.length, None)
 
     def test_tip_tip_distances_missing_length(self):
-        t = TreeNode.read(StringIO(u"((a,b:6)c:4,(d,e:0)f);"))
-        exp_t = TreeNode.read(StringIO(u"((a:0,b:6)c:4,(d:0,e:0)f:0);"))
+        t = TreeNode.read(io.StringIO("((a,b:6)c:4,(d,e:0)f);"))
+        exp_t = TreeNode.read(io.StringIO("((a:0,b:6)c:4,(d:0,e:0)f:0);"))
         exp_t_dm = exp_t.tip_tip_distances()
 
         t_dm = npt.assert_warns(RepresentationWarning, t.tip_tip_distances)
@@ -755,7 +793,7 @@ class TreeTests(TestCase):
 
     def test_neighbors(self):
         """Get neighbors of a node"""
-        t = TreeNode.read(StringIO(u"((a,b)c,(d,e)f);"))
+        t = TreeNode.read(io.StringIO("((a,b)c,(d,e)f);"))
         exp = t.children
         obs = t.neighbors()
         self.assertEqual(obs, exp)
@@ -774,7 +812,7 @@ class TreeTests(TestCase):
 
     def test_has_children(self):
         """Test if has children"""
-        t = TreeNode.read(StringIO(u"((a,b)c,(d,e)f);"))
+        t = TreeNode.read(io.StringIO("((a,b)c,(d,e)f);"))
         self.assertTrue(t.has_children())
         self.assertTrue(t.children[0].has_children())
         self.assertTrue(t.children[1].has_children())
@@ -817,9 +855,34 @@ class TreeTests(TestCase):
         obs = [n.name for n in self.simple_t.levelorder()]
         self.assertEqual(obs, exp)
 
+    def test_bifurcate(self):
+        t1 = TreeNode.read(io.StringIO('(((a,b),c),(d,e));'))
+        t2 = TreeNode.read(io.StringIO('((a,b,c));'))
+        t3 = t2.copy()
+
+        t1.bifurcate()
+        t2.bifurcate()
+        t3.bifurcate(insert_length=0)
+
+        self.assertEqual(str(t1), '(((a,b),c),(d,e));\n')
+        self.assertEqual(str(t2), '((c,(a,b)));\n')
+        self.assertEqual(str(t3), '((c,(a,b):0));\n')
+
+    def test_bifurcate_with_subclass(self):
+        tree = TreeNodeSubclass()
+        tree.append(TreeNodeSubclass())
+        tree.append(TreeNodeSubclass())
+        tree.append(TreeNodeSubclass())
+        tree.append(TreeNodeSubclass())
+
+        tree.bifurcate()
+
+        for node in tree.traverse():
+            self.assertIs(type(node), TreeNodeSubclass)
+
     def test_index_tree_single_node(self):
         """index_tree handles single node tree"""
-        t1 = TreeNode.read(StringIO(u'root;'))
+        t1 = TreeNode.read(io.StringIO('root;'))
         id_index, child_index = t1.index_tree()
         self.assertEqual(id_index[0], t1)
         npt.assert_equal(child_index, np.array([[]]))
@@ -827,9 +890,9 @@ class TreeTests(TestCase):
     def test_index_tree(self):
         """index_tree should produce correct index and node map"""
         # test for first tree: contains singleton outgroup
-        t1 = TreeNode.read(StringIO(u'(((a,b),c),(d,e));'))
-        t2 = TreeNode.read(StringIO(u'(((a,b),(c,d)),(e,f));'))
-        t3 = TreeNode.read(StringIO(u'(((a,b,c),(d)),(e,f));'))
+        t1 = TreeNode.read(io.StringIO('(((a,b),c),(d,e));'))
+        t2 = TreeNode.read(io.StringIO('(((a,b),(c,d)),(e,f));'))
+        t3 = TreeNode.read(io.StringIO('(((a,b,c),(d)),(e,f));'))
 
         id_1, child_1 = t1.index_tree()
         nodes_1 = [n.id for n in t1.traverse(self_before=False,
@@ -857,7 +920,7 @@ class TreeTests(TestCase):
 
     def test_root_at(self):
         """Form a new root"""
-        t = TreeNode.read(StringIO(u"(((a,b)c,(d,e)f)g,h)i;"))
+        t = TreeNode.read(io.StringIO("(((a,b)c,(d,e)f)g,h)i;"))
         with self.assertRaises(TreeError):
             t.root_at(t.find('h'))
 
@@ -881,16 +944,16 @@ class TreeTests(TestCase):
 
     def test_root_at_midpoint_no_lengths(self):
         # should get same tree back (a copy)
-        nwk = u'(a,b)c;\n'
-        t = TreeNode.read(StringIO(nwk))
+        nwk = '(a,b)c;\n'
+        t = TreeNode.read(io.StringIO(nwk))
         obs = t.root_at_midpoint()
         self.assertEqual(str(obs), nwk)
 
     def test_root_at_midpoint_tie(self):
-        nwk = u"(((a:1,b:1)c:2,(d:3,e:4)f:5),g:1)root;"
-        t = TreeNode.read(StringIO(nwk))
-        exp = u"((d:3,e:4)f:2,((a:1,b:1)c:2,(g:1)):3)root;"
-        texp = TreeNode.read(StringIO(exp))
+        nwk = "(((a:1,b:1)c:2,(d:3,e:4)f:5),g:1)root;"
+        t = TreeNode.read(io.StringIO(nwk))
+        exp = "((d:3,e:4)f:2,((a:1,b:1)c:2,(g:1)):3)root;"
+        texp = TreeNode.read(io.StringIO(exp))
 
         obs = t.root_at_midpoint()
 
@@ -900,9 +963,9 @@ class TreeTests(TestCase):
 
     def test_compare_subsets(self):
         """compare_subsets should return the fraction of shared subsets"""
-        t = TreeNode.read(StringIO(u'((H,G),(R,M));'))
-        t2 = TreeNode.read(StringIO(u'(((H,G),R),M);'))
-        t4 = TreeNode.read(StringIO(u'(((H,G),(O,R)),X);'))
+        t = TreeNode.read(io.StringIO('((H,G),(R,M));'))
+        t2 = TreeNode.read(io.StringIO('(((H,G),R),M);'))
+        t4 = TreeNode.read(io.StringIO('(((H,G),(O,R)),X);'))
 
         result = t.compare_subsets(t)
         self.assertEqual(result, 0)
@@ -927,9 +990,9 @@ class TreeTests(TestCase):
 
     def test_compare_rfd(self):
         """compare_rfd should return the Robinson Foulds distance"""
-        t = TreeNode.read(StringIO(u'((H,G),(R,M));'))
-        t2 = TreeNode.read(StringIO(u'(((H,G),R),M);'))
-        t4 = TreeNode.read(StringIO(u'(((H,G),(O,R)),X);'))
+        t = TreeNode.read(io.StringIO('((H,G),(R,M));'))
+        t2 = TreeNode.read(io.StringIO('(((H,G),R),M);'))
+        t4 = TreeNode.read(io.StringIO('(((H,G),(O,R)),X);'))
 
         obs = t.compare_rfd(t2)
         exp = 2.0
@@ -946,9 +1009,9 @@ class TreeTests(TestCase):
 
     def test_assign_ids(self):
         """Assign IDs to the tree"""
-        t1 = TreeNode.read(StringIO(u"(((a,b),c),(e,f),(g));"))
-        t2 = TreeNode.read(StringIO(u"(((a,b),c),(e,f),(g));"))
-        t3 = TreeNode.read(StringIO(u"((g),(e,f),(c,(a,b)));"))
+        t1 = TreeNode.read(io.StringIO("(((a,b),c),(e,f),(g));"))
+        t2 = TreeNode.read(io.StringIO("(((a,b),c),(e,f),(g));"))
+        t3 = TreeNode.read(io.StringIO("((g),(e,f),(c,(a,b)));"))
         t1_copy = t1.copy()
 
         t1.assign_ids()
@@ -965,9 +1028,9 @@ class TreeTests(TestCase):
 
     def test_assign_ids_index_tree(self):
         """assign_ids and index_tree should assign the same IDs"""
-        t1 = TreeNode.read(StringIO(u'(((a,b),c),(d,e));'))
-        t2 = TreeNode.read(StringIO(u'(((a,b),(c,d)),(e,f));'))
-        t3 = TreeNode.read(StringIO(u'(((a,b,c),(d)),(e,f));'))
+        t1 = TreeNode.read(io.StringIO('(((a,b),c),(d,e));'))
+        t2 = TreeNode.read(io.StringIO('(((a,b),(c,d)),(e,f));'))
+        t3 = TreeNode.read(io.StringIO('(((a,b,c),(d)),(e,f));'))
         t1_copy = t1.copy()
         t2_copy = t2.copy()
         t3_copy = t3.copy()
@@ -988,7 +1051,7 @@ class TreeTests(TestCase):
 
     def test_unrooted_deepcopy(self):
         """Do an unrooted_copy"""
-        t = TreeNode.read(StringIO(u"((a,(b,c)d)e,(f,g)h)i;"))
+        t = TreeNode.read(io.StringIO("((a,(b,c)d)e,(f,g)h)i;"))
         exp = "(b,c,(a,((f,g)h)e)d)root;\n"
         obs = t.find('d').unrooted_deepcopy()
         self.assertEqual(str(obs), exp)
@@ -1000,8 +1063,8 @@ class TreeTests(TestCase):
 
     def test_descending_branch_length(self):
         """Calculate descending branch_length"""
-        tr = TreeNode.read(StringIO(u"(((A:.1,B:1.2)C:.6,(D:.9,E:.6)F:.9)G:2.4"
-                                    ",(H:.4,I:.5)J:1.3)K;"))
+        tr = TreeNode.read(io.StringIO(
+            "(((A:.1,B:1.2)C:.6,(D:.9,E:.6)F:.9)G:2.4,(H:.4,I:.5)J:1.3)K;"))
         tdbl = tr.descending_branch_length()
         sdbl = tr.descending_branch_length(['A', 'E'])
         npt.assert_almost_equal(tdbl, 8.9)
@@ -1010,36 +1073,36 @@ class TreeTests(TestCase):
                           ['A', 'DNE'])
         self.assertRaises(ValueError, tr.descending_branch_length, ['A', 'C'])
 
-        tr = TreeNode.read(StringIO(u"(((A,B:1.2)C:.6,(D:.9,E:.6)F:.9)G:2.4,(H"
-                                    ":.4,I:.5)J:1.3)K;"))
+        tr = TreeNode.read(io.StringIO(
+            "(((A,B:1.2)C:.6,(D:.9,E:.6)F:.9)G:2.4,(H:.4,I:.5)J:1.3)K;"))
         tdbl = tr.descending_branch_length()
         npt.assert_almost_equal(tdbl, 8.8)
 
-        tr = TreeNode.read(StringIO(u"(((A,B:1.2)C:.6,(D:.9,E:.6)F)G:2.4,(H:.4"
-                                    ",I:.5)J:1.3)K;"))
+        tr = TreeNode.read(io.StringIO(
+            "(((A,B:1.2)C:.6,(D:.9,E:.6)F)G:2.4,(H:.4,I:.5)J:1.3)K;"))
         tdbl = tr.descending_branch_length()
         npt.assert_almost_equal(tdbl, 7.9)
 
-        tr = TreeNode.read(StringIO(u"(((A,B:1.2)C:.6,(D:.9,E:.6)F)G:2.4,(H:.4"
-                                    ",I:.5)J:1.3)K;"))
+        tr = TreeNode.read(io.StringIO(
+            "(((A,B:1.2)C:.6,(D:.9,E:.6)F)G:2.4,(H:.4,I:.5)J:1.3)K;"))
         tdbl = tr.descending_branch_length(['A', 'D', 'E'])
         npt.assert_almost_equal(tdbl, 2.1)
 
-        tr = TreeNode.read(StringIO(u"(((A,B:1.2)C:.6,(D:.9,E:.6)F:.9)G:2.4,(H"
-                                    ":.4,I:.5)J:1.3)K;"))
+        tr = TreeNode.read(io.StringIO(
+            "(((A,B:1.2)C:.6,(D:.9,E:.6)F:.9)G:2.4,(H:.4,I:.5)J:1.3)K;"))
         tdbl = tr.descending_branch_length(['I', 'D', 'E'])
         npt.assert_almost_equal(tdbl, 6.6)
 
         # test with a situation where we have unnamed internal nodes
-        tr = TreeNode.read(StringIO(u"(((A,B:1.2):.6,(D:.9,E:.6)F):2.4,(H:.4,I"
-                                    ":.5)J:1.3);"))
+        tr = TreeNode.read(io.StringIO(
+            "(((A,B:1.2):.6,(D:.9,E:.6)F):2.4,(H:.4,I:.5)J:1.3);"))
         tdbl = tr.descending_branch_length()
         npt.assert_almost_equal(tdbl, 7.9)
 
     def test_to_array(self):
         """Convert a tree to arrays"""
-        t = TreeNode.read(StringIO(
-            u'(((a:1,b:2,c:3)x:4,(d:5)y:6)z:7,(e:8,f:9)z:10);'))
+        t = TreeNode.read(io.StringIO(
+            '(((a:1,b:2,c:3)x:4,(d:5)y:6)z:7,(e:8,f:9)z:10);'))
         id_index, child_index = t.index_tree()
         arrayed = t.to_array()
 
@@ -1060,8 +1123,8 @@ class TreeTests(TestCase):
         npt.assert_equal(obs, exp)
 
     def test_to_array_attrs(self):
-        t = TreeNode.read(StringIO(
-            u'(((a:1,b:2,c:3)x:4,(d:5)y:6)z:7,(e:8,f:9)z:10);'))
+        t = TreeNode.read(io.StringIO(
+            '(((a:1,b:2,c:3)x:4,(d:5)y:6)z:7,(e:8,f:9)z:10);'))
         id_index, child_index = t.index_tree()
         arrayed = t.to_array(attrs=[('name', object)])
 
@@ -1082,7 +1145,7 @@ class TreeTests(TestCase):
             t.to_array(attrs=[('name', object), ('brofist', int)])
 
     def test_to_array_nan_length_value(self):
-        t = TreeNode.read(StringIO(u"((a:1, b:2)c:3)root;"))
+        t = TreeNode.read(io.StringIO("((a:1, b:2)c:3)root;"))
         indexed = t.to_array(nan_length_value=None)
         npt.assert_equal(indexed['length'],
                          np.array([1, 2, 3, np.nan], dtype=float))
@@ -1093,12 +1156,12 @@ class TreeTests(TestCase):
         npt.assert_equal(indexed['length'],
                          np.array([1, 2, 3, 42.0], dtype=float))
 
-        t = TreeNode.read(StringIO(u"((a:1, b:2)c:3)root:4;"))
+        t = TreeNode.read(io.StringIO("((a:1, b:2)c:3)root:4;"))
         indexed = t.to_array(nan_length_value=42.0)
         npt.assert_equal(indexed['length'],
                          np.array([1, 2, 3, 4], dtype=float))
 
-        t = TreeNode.read(StringIO(u"((a:1, b:2)c)root;"))
+        t = TreeNode.read(io.StringIO("((a:1, b:2)c)root;"))
         indexed = t.to_array(nan_length_value=42.0)
         npt.assert_equal(indexed['length'],
                          np.array([1, 2, 42.0, 42.0], dtype=float))
@@ -1109,13 +1172,20 @@ class TreeTests(TestCase):
                           '3': ['h', 'i', 'j', 'k', 'l', 'm', 'n'],
                           '4': ['h', 'i', 'j', 'k', 'l', 'm', 'q'],
                           '5': ['h', 'i', 'j', 'k', 'l', 'm', 'n']}
-        exp = TreeNode.read(StringIO(u"((((((((1)g)f)e)d,((((2)y)x)))c)b)a,"
-                                     "(((((((3,5)n,(4)q)m)l)k)j)i)h);"))
+        exp = TreeNode.read(io.StringIO(
+            "((((((((1)g)f)e)d,((((2)y)x)))c)b)a,"
+            "(((((((3,5)n,(4)q)m)l)k)j)i)h);"))
 
         root = TreeNode.from_taxonomy(input_lineages.items())
 
+        self.assertIs(type(root), TreeNode)
+
         self.assertEqual(root.compare_subsets(exp), 0.0)
 
+        root = TreeNodeSubclass.from_taxonomy(input_lineages.items())
+
+        self.assertIs(type(root), TreeNodeSubclass)
+
     def test_to_taxonomy(self):
         input_lineages = {'1': ['a', 'b', 'c', 'd', 'e', 'f', 'g'],
                           '2': ['a', 'b', 'c', None, None, 'x', 'y'],
@@ -1156,10 +1226,17 @@ class TreeTests(TestCase):
                               [4.0, 11.0, 34.0,  7.0]])
 
         tree = TreeNode.from_linkage_matrix(linkage, id_list)
+
+        self.assertIs(type(tree), TreeNode)
+
         self.assertEqual("(E:17.0,(C:14.5,((A:4.0,D:4.0):4.25,(G:6.25,(B:0.5,"
                          "F:0.5):5.75):2.0):6.25):2.5);\n",
                          str(tree))
 
+        tree = TreeNodeSubclass.from_linkage_matrix(linkage, id_list)
+
+        self.assertIs(type(tree), TreeNodeSubclass)
+
     def test_shuffle_invalid_iter(self):
         shuffler = self.simple_t.shuffle(n=-1)
         with self.assertRaises(ValueError):
